@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*- 
-//-----------------------------------------------------------------------------
-//
-// $Id: am_map.c,v 1.24 1998/05/10 12:05:24 jim Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 // DESCRIPTION:  
 //   the automap code
 //
@@ -73,10 +64,6 @@ int map_keyed_door_flash; // keyed doors are flashing
 
 int map_smooth_lines;
 
-//jff 4/3/98 add symbols for "no-color" for disable and "black color" for black
-#define NC 0
-#define BC 247
-
 // [Woof!] FRACTOMAPBITS: overflow-safe coordinate system.
 // Written by Andrey Budko (entryway), adapted from prboom-plus/src/am_map.*
 #define MAPBITS 12
@@ -110,11 +97,11 @@ int map_smooth_lines;
 #define M2_ZOOMOUTFAST  ((int) (FRACUNIT/1.5))
 
 // [crispy] toggleable pan/zoom speed
-static int f_paninc;
-static int m_zoomin_kbd;
-static int m_zoomout_kbd;
-static int m_zoomin_mouse;
-static int m_zoomout_mouse;
+static int f_paninc = F_PANINC;
+static int m_zoomin_kbd = M_ZOOMIN;
+static int m_zoomout_kbd = M_ZOOMOUT;
+static int m_zoomin_mouse = M2_ZOOMIN;
+static int m_zoomout_mouse = M2_ZOOMOUT;
 static boolean mousewheelzoom;
 
 // translates between frame-buffer and map distances
@@ -140,18 +127,13 @@ typedef struct
     mpoint_t a, b;
 } mline_t;
 
-typedef struct
-{
-    fixed_t slp, islp;
-} islope_t;
-
 //
 // The vector graphics for the automap.
 //  A line drawing of the player pointing right,
 //   starting from the middle.
 //
 #define R ((8*MAPPLAYERRADIUS)/7)
-mline_t player_arrow[] =
+static mline_t player_arrow[] =
 {
   { { -R+R/8, 0 }, { R, 0 } }, // -----
   { { R, 0 }, { R-R/2, R/4 } },  // ----->
@@ -165,7 +147,7 @@ mline_t player_arrow[] =
 #define NUMPLYRLINES (sizeof(player_arrow)/sizeof(mline_t))
 
 #define R ((8*MAPPLAYERRADIUS)/7)
-mline_t cheat_player_arrow[] =
+static mline_t cheat_player_arrow[] =
 { // killough 3/22/98: He's alive, Jim :)
   { { -R+R/8, 0 }, { R, 0 } }, // -----
   { { R, 0 }, { R-R/2, R/4 } },  // ----->
@@ -185,31 +167,9 @@ mline_t cheat_player_arrow[] =
 #undef R
 #define NUMCHEATPLYRLINES (sizeof(cheat_player_arrow)/sizeof(mline_t))
 
-#define R (FRACUNIT)
-
-#define np867R (int)(-.867*R)
-#define p867R  (int)(.867*R)
-#define np5R   (int)(-.5*R)
-#define p5R    (int)(.5*R)
-
-mline_t triangle_guy[] =
-{
-  { { np867R, np5R }, { p867R,  np5R } },
-  { { p867R,  np5R }, { 0,         R } },
-  { { 0,         R }, { np867R, np5R } }
-};
-
-#undef R
-#undef np867R 
-#undef p867R  
-#undef np5R   
-#undef p5R    
-
-#define NUMTRIANGLEGUYLINES (sizeof(triangle_guy)/sizeof(mline_t))
-
 //jff 1/5/98 new symbol for keys on automap
 #define R (FRACUNIT)
-mline_t cross_mark[] =
+static mline_t cross_mark[] =
 {
   { { -R, 0 }, { R, 0} },
   { { 0, -R }, { 0, R } },
@@ -229,7 +189,7 @@ static mline_t square_mark[] = {
 #define np7R (int)(-.7*R)
 #define p7R  (int)(.7*R)
 
-mline_t thintriangle_guy[] =
+static mline_t thintriangle_guy[] =
 {
   { { np5R, np7R }, { R,       0 } },
   { { R,       0 }, { np5R,  p7R } },
@@ -247,6 +207,7 @@ int ddt_cheating = 0;         // killough 2/7/98: make global, rename to ddt_*
 int automap_grid = 0;
 
 boolean automapactive = false;
+static boolean automapfirststart = true;
 
 overlay_t automapoverlay = overlay_off;
 
@@ -258,9 +219,7 @@ static int  f_y;
 static int  f_w;
 static int  f_h;
 
-static int  lightlev;        // used for funky strobing effect
 static byte*  fb;            // pseudo-frame buffer
-static int  amclock;
 
 static mpoint_t m_paninc;    // how far the window pans each tic (map coords)
 static fixed_t mtof_zoommul; // how far the window zooms each tic (map coords)
@@ -328,7 +287,7 @@ void (*AM_drawFline)(fline_t*, int) = AM_drawFline_Vanilla;
 
 // [crispy] automap rotate mode needs these early on
 boolean automaprotate = false;
-void AM_rotate(int64_t *x, int64_t *y, angle_t a);
+static void AM_rotate(int64_t *x, int64_t *y, angle_t a);
 static void AM_rotatePoint(mpoint_t *pt);
 static mpoint_t mapcenter;
 static angle_t mapangle;
@@ -337,34 +296,6 @@ static angle_t mapangle;
 extern int mousebprevweapon;
 extern int mousebnextweapon;
 
-//
-// AM_getIslope()
-//
-// Calculates the slope and slope according to the x-axis of a line
-// segment in map coordinates (with the upright y-axis n' all) so
-// that it can be used with the brain-dead drawing stuff.
-//
-// Passed the line slope is desired for and an islope_t structure for return
-// Returns nothing
-//
-void AM_getIslope
-( mline_t*  ml,
-  islope_t* is )
-{
-  int dx, dy;
-
-  dy = ml->a.y - ml->b.y;
-  dx = ml->b.x - ml->a.x;
-  if (!dy)
-    is->islp = (dx<0?-D_MAXINT:D_MAXINT);
-  else
-    is->islp = FixedDiv(dx, dy);
-  if (!dx)
-    is->slp = (dy<0?-D_MAXINT:D_MAXINT);
-  else
-    is->slp = FixedDiv(dy, dx);
-}
-
 //
 // AM_activateNewScale()
 //
@@ -372,7 +303,7 @@ void AM_getIslope
 //
 // Passed nothing, returns nothing
 //
-void AM_activateNewScale(void)
+static void AM_activateNewScale(void)
 {
   m_x += m_w/2;
   m_y += m_h/2;
@@ -392,7 +323,7 @@ void AM_activateNewScale(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_saveScaleAndLoc(void)
+static void AM_saveScaleAndLoc(void)
 {
   old_m_x = m_x;
   old_m_y = m_y;
@@ -408,7 +339,7 @@ void AM_saveScaleAndLoc(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_restoreScaleAndLoc(void)
+static void AM_restoreScaleAndLoc(void)
 {
   m_w = old_m_w;
   m_h = old_m_h;
@@ -438,7 +369,7 @@ void AM_restoreScaleAndLoc(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_addMark(void)
+static void AM_addMark(void)
 {
   // killough 2/22/98:
   // remove limit on automap marks
@@ -449,9 +380,8 @@ void AM_addMark(void)
                             markpointnum_max*2 : 16) * sizeof(*markpoints),
                            PU_STATIC, 0);
 
-  // [crispy] keep the map static in overlay mode
-  // if not following the player
-  if (!followplayer && automapoverlay)
+  // [crispy] keep the map static if not following the player
+  if (!followplayer)
   {
     markpoints[markpointnum].x = plr->mo->x >> FRACTOMAPBITS;
     markpoints[markpointnum].y = plr->mo->y >> FRACTOMAPBITS;
@@ -472,7 +402,7 @@ void AM_addMark(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_findMinMaxBoundaries(void)
+static void AM_findMinMaxBoundaries(void)
 {
   int i;
   fixed_t a;
@@ -508,6 +438,14 @@ void AM_findMinMaxBoundaries(void)
   max_scale_mtof = FixedDiv(f_h<<FRACBITS, 2*MAPPLAYERRADIUS);
 }
 
+void AM_SetMapCenter(fixed_t x, fixed_t y)
+{
+  m_x = (x >> FRACTOMAPBITS) - m_w / 2;
+  m_y = (y >> FRACTOMAPBITS) - m_h / 2;
+  m_x2 = m_x + m_w;
+  m_y2 = m_y + m_h;
+}
+
 //
 // AM_changeWindowLoc()
 //
@@ -515,7 +453,7 @@ void AM_findMinMaxBoundaries(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_changeWindowLoc(void)
+static void AM_changeWindowLoc(void)
 {
   int64_t incx, incy;
 
@@ -573,9 +511,6 @@ void AM_initVariables(void)
   automapactive = true;
   fb = screens[0];
 
-  amclock = 0;
-  lightlev = 0;
-
   m_paninc.x = m_paninc.y = 0;
   ftom_zoommul = FRACUNIT;
   mtof_zoommul = FRACUNIT;
@@ -585,8 +520,13 @@ void AM_initVariables(void)
   m_h = FTOM(f_h);
 
   plr = &players[displayplayer];
-  m_x = (plr->mo->x >> FRACTOMAPBITS) - m_w/2;
-  m_y = (plr->mo->y >> FRACTOMAPBITS) - m_h/2;
+  // [Alaux] Don't always snap back to player when reopening the Automap
+  if (followplayer || automapfirststart)
+  {
+    m_x = (plr->mo->x >> FRACTOMAPBITS) - m_w/2;
+    m_y = (plr->mo->y >> FRACTOMAPBITS) - m_h/2;
+    automapfirststart = false;
+  }
   AM_Ticker(); // initialize variables for interpolation
   AM_changeWindowLoc();
 
@@ -608,7 +548,7 @@ void AM_initVariables(void)
 // Sets the marknums[i] variables to the patches for each digit
 // Passed nothing, returns nothing;
 //
-void AM_loadPics(void)
+static void AM_loadPics(void)
 {
   int i;
   char namebuf[9];
@@ -627,7 +567,7 @@ void AM_loadPics(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_unloadPics(void)
+static void AM_unloadPics(void)
 {
   int i;
 
@@ -669,11 +609,13 @@ void AM_enableSmoothLines(void)
 // Passed nothing, returns nothing
 // Affects automap's global variables
 //
-void AM_LevelInit(void)
+static void AM_LevelInit(void)
 {
   // [crispy] Only need to precalculate color lookup tables once
   static int precalc_once;
 
+  automapfirststart = true;
+  
   f_x = f_y = 0;
 
   // killough 2/7/98: get rid of finit_ vars
@@ -682,7 +624,10 @@ void AM_LevelInit(void)
   // killough 11/98: ... finally add hires support :)
 
   f_w = (SCREENWIDTH) << hires;
-  f_h = (SCREENHEIGHT-ST_HEIGHT) << hires;
+  if (automapoverlay && scaledviewheight == SCREENHEIGHT)
+    f_h = (SCREENHEIGHT) << hires;
+  else
+    f_h = (SCREENHEIGHT-ST_HEIGHT) << hires;
 
   AM_enableSmoothLines();
 
@@ -751,16 +696,17 @@ void AM_Stop (void)
 //
 void AM_Start()
 {
-  static int lastlevel = -1, lastepisode = -1, last_hires = -1, last_widescreen = -1;
+  static int lastlevel = -1, lastepisode = -1, last_hires = -1, last_widescreen = -1, last_viewheight = -1;
 
   if (!stopped)
     AM_Stop();
   stopped = false;
   if (lastlevel != gamemap || lastepisode != gameepisode || hires!=last_hires
-    || widescreen != last_widescreen)
+    || widescreen != last_widescreen || viewheight != last_viewheight)
   {
     last_hires = hires;          // killough 11/98
     last_widescreen = widescreen;
+    last_viewheight = viewheight;
     AM_LevelInit();
     lastlevel = gamemap;
     lastepisode = gameepisode;
@@ -776,7 +722,7 @@ void AM_Start()
 //
 // Passed nothing, returns nothing
 //
-void AM_minOutWindowScale()
+static void AM_minOutWindowScale()
 {
   scale_mtof = min_scale_mtof;
   scale_ftom = FixedDiv(FRACUNIT, scale_mtof);
@@ -790,13 +736,26 @@ void AM_minOutWindowScale()
 //
 // Passed nothing, returns nothing
 //
-void AM_maxOutWindowScale(void)
+static void AM_maxOutWindowScale(void)
 {
   scale_mtof = max_scale_mtof;
   scale_ftom = FixedDiv(FRACUNIT, scale_mtof);
   AM_activateNewScale();
 }
 
+enum
+{
+  PAN_UP,
+  PAN_DOWN,
+  PAN_LEFT,
+  PAN_RIGHT,
+  ZOOM_IN,
+  ZOOM_OUT,
+  STATE_NUM
+};
+
+static int buttons_state[STATE_NUM] = { 0 };
+
 //
 // AM_Responder()
 //
@@ -809,9 +768,8 @@ boolean AM_Responder
 {
   int rc;
   static int bigstate=0;
-  static char buffer[20];
 
-  if (M_InputGameActive(input_speed))
+  if (M_InputActivated(input_speed))
   {
     f_paninc = F2_PANINC;
     m_zoomin_kbd = M2_ZOOMIN;
@@ -819,7 +777,7 @@ boolean AM_Responder
     m_zoomin_mouse = M2_ZOOMINFAST;
     m_zoomout_mouse = M2_ZOOMOUTFAST;
   }
-  else
+  else if (M_InputDeactivated(input_speed))
   {
     f_paninc = F_PANINC;
     m_zoomin_kbd = M_ZOOMIN;
@@ -847,24 +805,24 @@ boolean AM_Responder
                                                                 // phares
     if (M_InputActivated(input_map_right))                      //    |
       if (!followplayer)                                        //    V
-        m_paninc.x = FTOM(f_paninc);
+        buttons_state[PAN_RIGHT] = 1;
       else
         rc = false;
     else if (M_InputActivated(input_map_left))
       if (!followplayer)
-          m_paninc.x = -FTOM(f_paninc);
+        buttons_state[PAN_LEFT] = 1;
       else
-          rc = false;
+        rc = false;
     else if (M_InputActivated(input_map_up))
       if (!followplayer)
-          m_paninc.y = FTOM(f_paninc);
+        buttons_state[PAN_UP] = 1;
       else
-          rc = false;
+        rc = false;
     else if (M_InputActivated(input_map_down))
       if (!followplayer)
-          m_paninc.y = -FTOM(f_paninc);
+        buttons_state[PAN_DOWN] = 1;
       else
-          rc = false;
+        rc = false;
     else if (M_InputActivated(input_map_zoomout))
     {
       if (ev->type == ev_mouseb_down && M_IsMouseWheel(ev->data1))
@@ -874,10 +832,7 @@ boolean AM_Responder
         ftom_zoommul = m_zoomin_mouse;
       }
       else
-      {
-        mtof_zoommul = m_zoomout_kbd;
-        ftom_zoommul = m_zoomin_kbd;
-      }
+        buttons_state[ZOOM_OUT] = 1;
     }
     else if (M_InputActivated(input_map_zoomin))
     {
@@ -888,10 +843,7 @@ boolean AM_Responder
         ftom_zoommul = m_zoomout_mouse;
       }
       else
-      {
-        mtof_zoommul = m_zoomin_kbd;
-        ftom_zoommul = m_zoomout_kbd;
-      }
+        buttons_state[ZOOM_IN] = 1;
     }
     else if (M_InputActivated(input_map))
     {
@@ -913,32 +865,30 @@ boolean AM_Responder
     else if (M_InputActivated(input_map_follow))
     {
       followplayer = !followplayer;
-      m_paninc.x = 0;
-      m_paninc.y = 0;
+      memset(buttons_state, 0, sizeof(buttons_state));
       // Ty 03/27/98 - externalized
-      plr->message = followplayer ? s_AMSTR_FOLLOWON : s_AMSTR_FOLLOWOFF;  
+      displaymsg("%s", followplayer ? s_AMSTR_FOLLOWON : s_AMSTR_FOLLOWOFF);
     }
     else if (M_InputActivated(input_map_grid))
     {
       automap_grid = !automap_grid;      // killough 2/28/98
       // Ty 03/27/98 - *not* externalized
-      plr->message = automap_grid ? s_AMSTR_GRIDON : s_AMSTR_GRIDOFF;  
+      displaymsg("%s", automap_grid ? s_AMSTR_GRIDON : s_AMSTR_GRIDOFF);
     }
     else if (M_InputActivated(input_map_mark))
     {
       // Ty 03/27/98 - *not* externalized     
-      sprintf(buffer, "%s %d", s_AMSTR_MARKEDSPOT, markpointnum);  
-      plr->message = buffer;
+      displaymsg("%s %d", s_AMSTR_MARKEDSPOT, markpointnum);
       AM_addMark();
     }
     else if (M_InputActivated(input_map_clear))
     {
       // [Alaux] Clear just the last mark
       if (!markpointnum)
-        plr->message = s_AMSTR_MARKSCLEARED;
+        displaymsg("%s", s_AMSTR_MARKSCLEARED);
       else {
         AM_clearLastMark();
-        doomprintf("Cleared spot %d", markpointnum);
+        displaymsg("Cleared spot %d", markpointnum);
       }
     }
     else
@@ -949,18 +899,25 @@ boolean AM_Responder
 
       switch (automapoverlay)
       {
-        case 2:  plr->message = "Dark Overlay On";  break;
-        case 1:  plr->message = s_AMSTR_OVERLAYON;  break;
-        default: plr->message = s_AMSTR_OVERLAYOFF; break;
+        case 2:  displaymsg("Dark Overlay On");        break;
+        case 1:  displaymsg("%s", s_AMSTR_OVERLAYON);  break;
+        default: displaymsg("%s", s_AMSTR_OVERLAYOFF); break;
       }
+
+      if (automapoverlay && scaledviewheight == SCREENHEIGHT)
+        f_h = (SCREENHEIGHT) << hires;
+      else
+        f_h = (SCREENHEIGHT-ST_HEIGHT) << hires;
+
+      AM_activateNewScale();
     }
     else if (M_InputActivated(input_map_rotate))
     {
       automaprotate = !automaprotate;
       if (automaprotate)
-        plr->message = s_AMSTR_ROTATEON;
+        displaymsg("%s", s_AMSTR_ROTATEON);
       else
-        plr->message = s_AMSTR_ROTATEOFF;
+        displaymsg("%s", s_AMSTR_ROTATEOFF);
     }
     else
     {
@@ -976,33 +933,66 @@ boolean AM_Responder
     if (M_InputDeactivated(input_map_right))
     {
       if (!followplayer)
-          m_paninc.x = 0;
+        buttons_state[PAN_RIGHT] = 0;
     }
     else if (M_InputDeactivated(input_map_left))
     {
       if (!followplayer)
-          m_paninc.x = 0;
+        buttons_state[PAN_LEFT] = 0;
     }
     else if (M_InputDeactivated(input_map_up))
     {
       if (!followplayer)
-          m_paninc.y = 0;
+        buttons_state[PAN_UP] = 0;
     }
     else if (M_InputDeactivated(input_map_down))
     {
       if (!followplayer)
-          m_paninc.y = 0;
+        buttons_state[PAN_DOWN] = 0;
     }
-    else if (M_InputDeactivated(input_map_zoomout) ||
-             M_InputDeactivated(input_map_zoomin))
+    else if (M_InputDeactivated(input_map_zoomout))
     {
-      if (!mousewheelzoom)
-      {
-        mtof_zoommul = FRACUNIT;
-        ftom_zoommul = FRACUNIT;
-      }
+      buttons_state[ZOOM_OUT] = 0;
+    }
+    else if (M_InputDeactivated(input_map_zoomin))
+    {
+      buttons_state[ZOOM_IN] = 0;
     }
   }
+
+  m_paninc.x = 0;
+  m_paninc.y = 0;
+
+  if (!followplayer)
+  {
+    if (buttons_state[PAN_RIGHT])
+      m_paninc.x += FTOM(f_paninc << hires);
+    if (buttons_state[PAN_LEFT])
+      m_paninc.x += -FTOM(f_paninc << hires);
+
+    if (buttons_state[PAN_UP])
+      m_paninc.y += FTOM(f_paninc << hires);
+    if (buttons_state[PAN_DOWN])
+      m_paninc.y += -FTOM(f_paninc << hires);
+  }
+
+  if (!mousewheelzoom)
+  {
+    mtof_zoommul = FRACUNIT;
+    ftom_zoommul = FRACUNIT;
+
+    if (buttons_state[ZOOM_OUT] && !buttons_state[ZOOM_IN])
+    {
+      mtof_zoommul = m_zoomout_kbd;
+      ftom_zoommul = m_zoomin_kbd;
+    }
+    else if (buttons_state[ZOOM_IN] && !buttons_state[ZOOM_OUT])
+    {
+      mtof_zoommul = m_zoomin_kbd;
+      ftom_zoommul = m_zoomout_kbd;
+    }
+  }
+
   return rc;
 }
 
@@ -1013,7 +1003,7 @@ boolean AM_Responder
 //
 // Passed nothing, returns nothing
 //
-void AM_changeWindowScale(void)
+static void AM_changeWindowScale(void)
 {
   // Change the scaling multipliers
   scale_mtof = FixedMul(scale_mtof, mtof_zoommul);
@@ -1042,7 +1032,7 @@ void AM_changeWindowScale(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_doFollowPlayer(void)
+static void AM_doFollowPlayer(void)
 {
   m_x = (viewx >> FRACTOMAPBITS) - m_w/2;
   m_y = (viewy >> FRACTOMAPBITS) - m_h/2;
@@ -1059,8 +1049,8 @@ int map_point_coordinates;
 
 void AM_Coordinates(const mobj_t *mo, fixed_t *x, fixed_t *y, fixed_t *z)
 {
-  *z = followplayer || !map_point_coordinates ? *x = mo->x, *y = mo->y, mo->z :
-    R_PointInSubsector(*x = m_x+m_w/2, *y = m_y+m_h/2)->sector->floorheight;
+  *z = followplayer || !map_point_coordinates || !automapactive ? *x = mo->x, *y = mo->y, mo->z :
+    R_PointInSubsector(*x = (m_x+m_w/2) << FRACTOMAPBITS, *y = (m_y+m_h/2) << FRACTOMAPBITS)->sector->floorheight;
 }
 
 //
@@ -1075,8 +1065,6 @@ void AM_Ticker (void)
   if (!automapactive)
     return;
 
-  amclock++;
-
   // Change the zoom if necessary.
   if (ftom_zoommul != FRACUNIT)
   {
@@ -1091,7 +1079,7 @@ void AM_Ticker (void)
 //
 // Clear automap frame buffer.
 //
-void AM_clearFB(int color)
+static void AM_clearFB(int color)
 {
   memset(fb, color, f_w*f_h);
 }
@@ -1109,7 +1097,7 @@ void AM_clearFB(int color)
 // clipping on them in the lines frame coordinates.
 // Returns true if any part of line was not clipped
 //
-boolean AM_clipMline
+static boolean AM_clipMline
 ( mline_t*  ml,
   fline_t*  fl )
 {
@@ -1258,8 +1246,6 @@ static void AM_drawFline_Vanilla(fline_t* fl, int color)
   register int d;
 
 #ifdef RANGECHECK         // killough 2/22/98    
-  static int fuck = 0;
-
   // For debugging only
   if
   (
@@ -1269,7 +1255,6 @@ static void AM_drawFline_Vanilla(fline_t* fl, int color)
     || fl->b.y < 0 || fl->b.y >= f_h
   )
   {
-    fprintf(stderr, "fuck %d \r", fuck++);
     return;
   }
 #endif
@@ -1466,7 +1451,7 @@ static void AM_drawFline_Smooth(fline_t* fl, int color)
 // in the defaults file.
 // Returns nothing.
 //
-void AM_drawMline
+static void AM_drawMline
 ( mline_t*  ml,
   int   color )
 {
@@ -1489,7 +1474,7 @@ void AM_drawMline
 // Passed the color to draw the grid lines
 // Returns nothing
 //
-void AM_drawGrid(int color)
+static void AM_drawGrid(int color)
 {
   int64_t x, y;
   int64_t start, end;
@@ -1579,7 +1564,7 @@ void AM_drawGrid(int color)
 //
 // jff 4/3/98 add routine to get color of generalized keyed door
 //
-int AM_DoorColor(int type)
+static int AM_DoorColor(int type)
 {
   if (GenLockedBase <= type && type< GenDoorBase)
   {
@@ -1622,7 +1607,7 @@ int AM_DoorColor(int type)
 // jff 4/3/98 changed mapcolor_xxxx=0 as control to disable feature
 // jff 4/3/98 changed mapcolor_xxxx=-1 to disable drawing line completely
 //
-void AM_drawWalls(void)
+static void AM_drawWalls(void)
 {
   int i;
   static mline_t l;
@@ -1838,20 +1823,22 @@ void AM_drawWalls(void)
 // Passed the coordinates of a point, and an angle
 // Returns the coordinates rotated by the angle
 //
-void AM_rotate
+static void AM_rotate
 ( int64_t*  x,
   int64_t*  y,
   angle_t a )
 {
   int64_t tmpx;
 
+  a >>= ANGLETOFINESHIFT;
+
   tmpx =
-    FixedMul(*x,finecosine[a>>ANGLETOFINESHIFT])
-      - FixedMul(*y,finesine[a>>ANGLETOFINESHIFT]);
+    FixedMul(*x,finecosine[a])
+      - FixedMul(*y,finesine[a]);
 
   *y   =
-    FixedMul(*x,finesine[a>>ANGLETOFINESHIFT])
-      + FixedMul(*y,finecosine[a>>ANGLETOFINESHIFT]);
+    FixedMul(*x,finesine[a])
+      + FixedMul(*y,finecosine[a]);
 
   *x = tmpx;
 }
@@ -1862,17 +1849,19 @@ static void AM_rotatePoint(mpoint_t *pt)
 {
   int64_t tmpx;
   // [crispy] smooth automap rotation
-  const angle_t smoothangle = (followplayer || !automapoverlay) ? ANG90 - viewangle : mapangle;
+  angle_t smoothangle = followplayer ? ANG90 - viewangle : mapangle;
 
   pt->x -= mapcenter.x;
   pt->y -= mapcenter.y;
 
-  tmpx = (int64_t)FixedMul(pt->x, finecosine[smoothangle>>ANGLETOFINESHIFT])
-       - (int64_t)FixedMul(pt->y, finesine[smoothangle>>ANGLETOFINESHIFT])
+  smoothangle >>= ANGLETOFINESHIFT;
+
+  tmpx = (int64_t)FixedMul(pt->x, finecosine[smoothangle])
+       - (int64_t)FixedMul(pt->y, finesine[smoothangle])
        + mapcenter.x;
 
-  pt->y = (int64_t)FixedMul(pt->x, finesine[smoothangle>>ANGLETOFINESHIFT])
-        + (int64_t)FixedMul(pt->y, finecosine[smoothangle>>ANGLETOFINESHIFT])
+  pt->y = (int64_t)FixedMul(pt->x, finesine[smoothangle])
+        + (int64_t)FixedMul(pt->y, finecosine[smoothangle])
         + mapcenter.y;
 
   pt->x = tmpx;
@@ -1888,7 +1877,7 @@ static void AM_rotatePoint(mpoint_t *pt)
 // the color to draw it with, and the map coordinates to draw it at.
 // Returns nothing
 //
-void AM_drawLineCharacter
+static void AM_drawLineCharacter
 ( mline_t*  lineguy,
   int   lineguylines,
   fixed_t scale,
@@ -1949,7 +1938,7 @@ void AM_drawLineCharacter
 //
 // Passed nothing, returns nothing
 //
-void AM_drawPlayers(void)
+static void AM_drawPlayers(void)
 {
   int   i;
   player_t* p;
@@ -2065,7 +2054,7 @@ void AM_drawPlayers(void)
 // Passed colors and colorrange, no longer used
 // Returns nothing
 //
-void AM_drawThings
+static void AM_drawThings
 ( int colors,
   int  colorrange)
 {
@@ -2212,7 +2201,7 @@ void AM_drawThings
 //
 // killough 11/98: added hires support
 
-void AM_drawMarks(void)
+static void AM_drawMarks(void)
 {
   int i;
   mpoint_t pt;
@@ -2262,7 +2251,7 @@ void AM_drawMarks(void)
 // Passed the color to draw the pixel with
 // Returns nothing
 //
-void AM_drawCrosshair(int color)
+static void AM_drawCrosshair(int color)
 {
   // [crispy] do not draw the useless dot on the player arrow
   if (!followplayer)
@@ -2301,9 +2290,8 @@ void AM_Drawer (void)
   {
     mapcenter.x = m_x + m_w / 2;
     mapcenter.y = m_y + m_h / 2;
-    // [crispy] keep the map static in overlay mode
-    // if not following the player
-    if (followplayer || !automapoverlay)
+    // [crispy] keep the map static if not following the player
+    if (followplayer)
     {
       mapangle = ANG90 - plr->mo->angle;
     }
