@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*- 
-//-----------------------------------------------------------------------------
-//
-// $Id: f_finale.c,v 1.16 1998/05/10 23:39:25 killough Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 // DESCRIPTION:
 //      Game completion, final screen animation.
 //
@@ -353,7 +344,6 @@ void F_Ticker(void)
 // written.                                                         // phares
 
 #include "hu_stuff.h"
-extern  patch_t *hu_font[HU_FONTSIZE];
 
 // [FG] add line breaks for lines exceeding screenwidth
 static inline boolean F_AddLineBreak (char *c)
@@ -852,10 +842,6 @@ void F_Drawer (void)
     if (!finalestage)
     {
       F_TextWrite();
-      if (!gamemapinfo->endpic[0])
-      {
-        using_FMI = false;
-      }
     }
     else if (strcmp(gamemapinfo->endpic, "$BUNNY") == 0)
     {
