@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*- 
-//-----------------------------------------------------------------------------
-//
-// $Id: m_cheat.c,v 1.7 1998/05/12 12:47:00 phares Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 // DESCRIPTION:
 //      Cheat sequence checking.
 //
@@ -43,6 +34,8 @@
 #include "m_misc2.h"
 #include "p_spec.h" // SPECHITS
 #include "d_main.h"
+#include "m_input.h"
+#include "am_map.h"
 
 #define plyr (players+consoleplayer)     /* the console player */
 
@@ -66,6 +59,8 @@ static void cheat_clev0();
 static void cheat_mypos();
 static void cheat_comp(char *buf);
 static void cheat_comp0();
+static void cheat_skill(char *buf);
+static void cheat_skill0();
 static void cheat_friction();
 static void cheat_pushers();
 static void cheat_tran();
@@ -87,6 +82,12 @@ static void cheat_rate();
 static void cheat_buddha();
 static void cheat_spechits();
 static void cheat_notarget();
+static void cheat_freeze();
+static void cheat_health();
+static void cheat_megaarmour();
+static void cheat_reveal_secret();
+static void cheat_reveal_kill();
+static void cheat_reveal_item();
 
 static void cheat_autoaim();      // killough 7/19/98
 static void cheat_tst();
@@ -144,6 +145,12 @@ struct cheat_s cheat[] = {
   {"idbeholdo",  NULL,                not_net | not_demo | not_deh,
    {cheat_pw}, NUMPOWERS }, // [FG] disable all powerups at once
 
+  {"idbeholdh",  "Health",            not_net | not_demo,
+   {cheat_health} },
+
+  {"idbeholdm",  "Mega Armor",        not_net | not_demo,
+   {cheat_megaarmour} },
+
   {"idbeholdv",  "Invincibility",     not_net | not_demo,
    {cheat_pw}, pw_invulnerability },
 
@@ -156,10 +163,10 @@ struct cheat_s cheat[] = {
   {"idbeholdr",  "Radiation Suit",    not_net | not_demo,
    {cheat_pw}, pw_ironfeet },
 
-  {"idbeholda",  "Auto-map",          not_net | not_demo,
+  {"idbeholda",  "Auto-map",          not_dm,
    {cheat_pw}, pw_allmap },
 
-  {"idbeholdl",  "Lite-Amp Goggles",  not_net | not_demo,
+  {"idbeholdl",  "Lite-Amp Goggles",  not_dm,
    {cheat_pw}, pw_infrared },
 
   {"idbehold",   "BEHOLD menu",       not_net | not_demo,
@@ -180,6 +187,12 @@ struct cheat_s cheat[] = {
   {"comp",    NULL,                   not_net | not_demo | not_menu,
    {cheat_comp0} },
 
+  {"skill",    NULL,                  not_net | not_demo | not_menu,
+   {cheat_skill}, -1 },
+
+  {"skill",    NULL,                  not_net | not_demo | not_menu,
+   {cheat_skill0} },
+
   {"killem",     NULL,                not_net | not_demo,
    {cheat_massacre} },   // jff 2/01/98 kill all monsters
 
@@ -189,9 +202,21 @@ struct cheat_s cheat[] = {
   {"notarget",   "Notarget mode",     not_net | not_demo,
    {cheat_notarget} },
 
+  {"freeze",     "Freeze",            not_net | not_demo,
+   {cheat_freeze} },
+
   {"iddt",       "Map cheat",         not_dm,
    {cheat_ddt} },        // killough 2/07/98: moved from am_map.c
 
+  {"iddst",      NULL,                always,
+   {cheat_reveal_secret} },
+
+  {"iddkt",      NULL,                not_dm,
+   {cheat_reveal_kill} },
+
+  {"iddit",      NULL,                not_dm,
+   {cheat_reveal_item} },
+
   {"hom",     NULL,                   always,
    {cheat_hom} },        // killough 2/07/98: HOM autodetector
 
@@ -299,9 +324,9 @@ static void cheat_showfps()
 static void cheat_autoaim()
 {
   extern int autoaim;
-  plyr->message = (autoaim=!autoaim) ?
+  displaymsg((autoaim=!autoaim) ?
     "Projectile autoaiming on" : 
-    "Projectile autoaiming off";
+    "Projectile autoaiming off");
 }
 
 static void cheat_mus(char *buf)
@@ -315,7 +340,7 @@ static void cheat_mus(char *buf)
   if (!isdigit(buf[0]) || !isdigit(buf[1]))
     return;
 
-  plyr->message = s_STSTR_MUS; // Ty 03/27/98 - externalized
+  displaymsg("%s", s_STSTR_MUS); // Ty 03/27/98 - externalized
   
   // First check if we have a mapinfo entry for the requested level.
   if (gamemode == commercial)
@@ -328,7 +353,7 @@ static void cheat_mus(char *buf)
      musnum = W_CheckNumForName(entry->music);
 
      if (musnum == -1)
-        plyr->message = s_STSTR_NOMUS;
+        displaymsg("%s", s_STSTR_NOMUS);
      else
      {
         S_ChangeMusInfoMusic(musnum, 1);
@@ -343,7 +368,7 @@ static void cheat_mus(char *buf)
           
       //jff 4/11/98 prevent IDMUS00 in DOOMII and IDMUS36 or greater
       if (musnum < mus_runnin || musnum >= NUMMUSIC)
-        plyr->message = s_STSTR_NOMUS; // Ty 03/27/98 - externalized
+        displaymsg("%s", s_STSTR_NOMUS); // Ty 03/27/98 - externalized
       else
         {
           S_ChangeMusic(musnum, 1);
@@ -356,7 +381,7 @@ static void cheat_mus(char *buf)
           
       //jff 4/11/98 prevent IDMUS0x IDMUSx0 in DOOMI and greater than introa
       if (musnum < mus_e1m1 || musnum >= mus_runnin)
-        plyr->message = s_STSTR_NOMUS; // Ty 03/27/98 - externalized
+        displaymsg("%s", s_STSTR_NOMUS); // Ty 03/27/98 - externalized
       else
         {
           S_ChangeMusic(musnum, 1);
@@ -370,7 +395,7 @@ static void cheat_choppers()
 {
   plyr->weaponowned[wp_chainsaw] = true;
   plyr->powers[pw_invulnerability] = true;
-  plyr->message = s_STSTR_CHOPPERS; // Ty 03/27/98 - externalized
+  displaymsg("%s", s_STSTR_CHOPPERS); // Ty 03/27/98 - externalized
 }
 
 static void cheat_god()
@@ -403,34 +428,72 @@ static void cheat_god()
         plyr->mo->health = god_health;  // Ty 03/09/98 - deh
           
       plyr->health = god_health;
-      plyr->message = s_STSTR_DQDON; // Ty 03/27/98 - externalized
+      displaymsg("%s", s_STSTR_DQDON); // Ty 03/27/98 - externalized
     }
   else 
-    plyr->message = s_STSTR_DQDOFF; // Ty 03/27/98 - externalized
+    displaymsg("%s", s_STSTR_DQDOFF); // Ty 03/27/98 - externalized
 }
 
 static void cheat_buddha()
 {
   plyr->cheats ^= CF_BUDDHA;
   if (plyr->cheats & CF_BUDDHA)
-    plyr->message = "Buddha Mode ON";
+    displaymsg("Buddha Mode ON");
   else
-    plyr->message = "Buddha Mode OFF";
+    displaymsg("Buddha Mode OFF");
 }
 
 static void cheat_notarget()
 {
   plyr->cheats ^= CF_NOTARGET;
   if (plyr->cheats & CF_NOTARGET)
-    plyr->message = "Notarget Mode ON";
+    displaymsg("Notarget ON");
   else
-    plyr->message = "Notarget Mode OFF";
+    displaymsg("Notarget OFF");
+}
+
+boolean frozen_mode;
+
+static void cheat_freeze()
+{
+  frozen_mode = !frozen_mode;
+  if (frozen_mode)
+    displaymsg("Freeze ON");
+  else
+    displaymsg("Freeze OFF");
+}
+
+static void cheat_avj()
+{
+  void A_VileJump(mobj_t *mo);
+
+  if (plyr->mo)
+    A_VileJump(plyr->mo);
+}
+
+// CPhipps - new health and armour cheat codes
+static void cheat_health()
+{
+  if (!(plyr->cheats & CF_GODMODE))
+  {
+    if (plyr->mo)
+      plyr->mo->health = mega_health;
+    plyr->health = mega_health;
+    displaymsg("%s", s_STSTR_BEHOLDX); // Ty 03/27/98 - externalized
+  }
+}
+
+static void cheat_megaarmour()
+{
+  plyr->armorpoints = idfa_armor;      // Ty 03/09/98 - deh
+  plyr->armortype = idfa_armor_class;  // Ty 03/09/98 - deh
+  displaymsg("%s", s_STSTR_BEHOLDX); // Ty 03/27/98 - externalized
 }
 
 static void cheat_tst()
 { // killough 10/98: same as iddqd except for message
   cheat_god();
-  plyr->message = plyr->cheats & CF_GODMODE ? "God Mode On" : "God Mode Off";
+  displaymsg(plyr->cheats & CF_GODMODE ? "God Mode On" : "God Mode Off");
 }
 
 static void cheat_fa()
@@ -457,7 +520,7 @@ static void cheat_fa()
     if (i!=am_cell || gamemode!=shareware)
       plyr->ammo[i] = plyr->maxammo[i];
 
-  plyr->message = s_STSTR_FAADDED;
+  displaymsg("%s", s_STSTR_FAADDED);
 }
 
 static void cheat_k()
@@ -467,7 +530,7 @@ static void cheat_k()
     if (!plyr->cards[i])     // only print message if at least one key added
       {                      // however, caller may overwrite message anyway
         plyr->cards[i] = true;
-        plyr->message = "Keys Added";
+        displaymsg("Keys Added");
       }
 }
 
@@ -475,7 +538,7 @@ static void cheat_kfa()
 {
   cheat_k();
   cheat_fa();
-  plyr->message = s_STSTR_KFAADDED;
+  displaymsg("%s", s_STSTR_KFAADDED);
 }
 
 static void cheat_noclip()
@@ -483,8 +546,8 @@ static void cheat_noclip()
   // Simplified, accepting both "noclip" and "idspispopd".
   // no clipping mode cheat
 
-  plyr->message = (plyr->cheats ^= CF_NOCLIP) & CF_NOCLIP ? 
-    s_STSTR_NCON : s_STSTR_NCOFF; // Ty 03/27/98 - externalized
+  displaymsg("%s", (plyr->cheats ^= CF_NOCLIP) & CF_NOCLIP ? 
+    s_STSTR_NCON : s_STSTR_NCOFF); // Ty 03/27/98 - externalized
 }
 
 // 'behold?' power-up cheats (modified for infinite duration -- killough)
@@ -504,13 +567,13 @@ static void cheat_pw(int pw)
       if (pw != pw_strength && !comp[comp_infcheat])
         plyr->powers[pw] = -1;      // infinite duration -- killough
     }
-  plyr->message = s_STSTR_BEHOLDX; // Ty 03/27/98 - externalized
+  displaymsg("%s", s_STSTR_BEHOLDX); // Ty 03/27/98 - externalized
 }
 
 // 'behold' power-up menu
 static void cheat_behold()
 {
-  plyr->message = s_STSTR_BEHOLD; // Ty 03/27/98 - externalized
+  displaymsg("%s", s_STSTR_BEHOLD); // Ty 03/27/98 - externalized
 }
 
 // 'clev' change-level cheat
@@ -526,9 +589,9 @@ static void cheat_clev0()
   next = MAPNAME(epsd, map);
 
   if (W_CheckNumForName(next) != -1)
-    doomprintf("Current: %s, Next: %s", cur, next);
+    displaymsg("Current: %s, Next: %s", cur, next);
   else
-    doomprintf("Current: %s", cur);
+    displaymsg("Current: %s", cur);
 
   free(cur);
 }
@@ -539,15 +602,15 @@ static void cheat_clev(char *buf)
   mapentry_t* entry;
 
   if (gamemode == commercial)
-    {
-      epsd = 1; //jff was 0, but espd is 1-based 
-      map = (buf[0] - '0')*10 + buf[1] - '0';
-    }
+  {
+    epsd = 1; //jff was 0, but espd is 1-based
+    map = (buf[0] - '0')*10 + buf[1] - '0';
+  }
   else
-    {
-      epsd = buf[0] - '0';
-      map = buf[1] - '0';
-    }
+  {
+    epsd = buf[0] - '0';
+    map = buf[1] - '0';
+  }
 
   // catch non-numerical input
   if (epsd < 0 || epsd > 9 || map < 0 || map > 99)
@@ -558,34 +621,37 @@ static void cheat_clev(char *buf)
   entry = G_LookupMapinfo(epsd, map);
   if (!entry)
   {
-  char *next = MAPNAME(epsd, map);
-
-  if (W_CheckNumForName(next) == -1)
-  {
-  // Catch invalid maps.
-  if (epsd < 1 || map < 1 ||   // Ohmygod - this is not going to work.
-      (gamemode == retail     && (epsd > 4 || map > 9  )) ||
-      (gamemode == registered && (epsd > 3 || map > 9  )) ||
-      (gamemode == shareware  && (epsd > 1 || map > 9  )) ||
-      (gamemode == commercial && (epsd > 1 || map > 32 )) )
-  {
-    doomprintf("IDCLEV target not found: %s", next);
-    return;
-  }
-  }
+    char *next;
 
     // Chex.exe always warps to episode 1.
     if (gameversion == exe_chex)
     {
       epsd = 1;
     }
+
+    next = MAPNAME(epsd, map);
+
+    if (W_CheckNumForName(next) == -1)
+    {
+      // [Alaux] Restart map with IDCLEV00
+      if ((epsd == 0 && map == 0) || (gamemode == commercial && map == 0))
+      {
+        epsd = gameepisode;
+        map = gamemap;
+      }
+      else
+      {
+        displaymsg("IDCLEV target not found: %s", next);
+        return;
+      }
+    }
   }
 
   // So be it.
 
   idmusnum = -1; //jff 3/17/98 revert to normal level music on IDCLEV
 
-  plyr->message = s_STSTR_CLEV; // Ty 03/27/98 - externalized
+  displaymsg("%s", s_STSTR_CLEV); // Ty 03/27/98 - externalized
 
   G_DeferedInitNew(gameskill, epsd, map);
 }
@@ -601,7 +667,7 @@ static void cheat_mypos()
 
 void cheat_mypos_print()
 {
-  doomprintf("X=%.10f Y=%.10f A=%-.0f",
+  displaymsg("X=%.10f Y=%.10f A=%-.0f",
           (double)players[consoleplayer].mo->x / FRACUNIT,
           (double)players[consoleplayer].mo->y / FRACUNIT,
           players[consoleplayer].mo->angle * (90.0/ANG90));
@@ -611,7 +677,7 @@ void cheat_mypos_print()
 
 static void cheat_comp0()
 {
-  doomprintf("Complevel: %s", G_GetCurrentComplevelName());
+  displaymsg("Complevel: %s", G_GetCurrentComplevelName());
 }
 
 static void cheat_comp(char *buf)
@@ -629,32 +695,52 @@ static void cheat_comp(char *buf)
   {
     demo_version = new_demover;
     G_ReloadDefaults(true);
-    doomprintf("New Complevel: %s", G_GetCurrentComplevelName());
+    displaymsg("New Complevel: %s", G_GetCurrentComplevelName());
   }
 }
 
 // variable friction cheat
 static void cheat_friction()
 {
-  plyr->message =                       // Ty 03/27/98 - *not* externalized
+  displaymsg(            // Ty 03/27/98 - *not* externalized
     (variable_friction = !variable_friction) ? "Variable Friction enabled" : 
-                                               "Variable Friction disabled";
+                                               "Variable Friction disabled");
 }
 
+extern const char *default_skill_strings[];
+
+static void cheat_skill0()
+{
+  displaymsg("Skill: %s", default_skill_strings[gameskill + 1]);
+}
+
+static void cheat_skill(char *buf)
+{
+  int skill = buf[0] - '0';
+
+  if (skill >= 1 && skill <= 5)
+  {
+    gameskill = skill - 1;
+    displaymsg("Next Level Skill: %s", default_skill_strings[gameskill + 1]);
+
+    G_SetFastParms(fastparm || gameskill == sk_nightmare);
+    respawnmonsters = gameskill == sk_nightmare || respawnparm;
+  }
+}
 
 // Pusher cheat
 // phares 3/10/98
 static void cheat_pushers()
 {
-  plyr->message =                      // Ty 03/27/98 - *not* externalized
-    (allow_pushers = !allow_pushers) ? "Pushers enabled" : "Pushers disabled";
+  displaymsg(           // Ty 03/27/98 - *not* externalized
+    (allow_pushers = !allow_pushers) ? "Pushers enabled" : "Pushers disabled");
 }
 
 // translucency cheat
 static void cheat_tran()
 {
-  plyr->message =                      // Ty 03/27/98 - *not* externalized
-    (translucency = !translucency) ? "Translucency enabled" : "Translucency disabled";
+  displaymsg(            // Ty 03/27/98 - *not* externalized
+    (translucency = !translucency) ? "Translucency enabled" : "Translucency disabled");
   D_SetPredefinedTranslucency();
 }
 
@@ -694,7 +780,7 @@ static void cheat_massacre()    // jff 2/01/98 kill all monsters
   P_MapEnd();
   // killough 3/22/98: make more intelligent about plural
   // Ty 03/27/98 - string(s) *not* externalized
-  doomprintf("%d Monster%s Killed", killcount, killcount==1 ? "" : "s");
+  displaymsg("%d Monster%s Killed", killcount, killcount==1 ? "" : "s");
 }
 
 static void cheat_spechits()
@@ -840,7 +926,7 @@ static void cheat_spechits()
     speciallines += EV_DoDoor(&dummy, doorOpen);
   }
 
-  doomprintf("%d Special Action%s Triggered", speciallines, speciallines == 1 ? "" : "s");
+  displaymsg("%d Special Action%s Triggered", speciallines, speciallines == 1 ? "" : "s");
 }
 
 // killough 2/7/98: move iddt cheat from am_map.c to here
@@ -852,45 +938,144 @@ static void cheat_ddt()
     ddt_cheating = (ddt_cheating+1) % 3;
 }
 
+static void cheat_reveal_secret()
+{
+  static int last_secret = -1;
+
+  if (automapactive)
+  {
+    int i, start_i;
+
+    i = last_secret + 1;
+    if (i >= numsectors)
+      i = 0;
+    start_i = i;
+
+    do
+    {
+      sector_t *sec = &sectors[i];
+
+      if (P_IsSecret(sec))
+      {
+        followplayer = false;
+
+        // This is probably not necessary
+        if (sec->lines && sec->lines[0] && sec->lines[0]->v1)
+        {
+          AM_SetMapCenter(sec->lines[0]->v1->x, sec->lines[0]->v1->y);
+          last_secret = i;
+          break;
+        }
+      }
+
+      i++;
+      if (i >= numsectors)
+        i = 0;
+    } while (i != start_i);
+  }
+}
+
+static void cheat_cycle_mobj(mobj_t **last_mobj, int *last_count,
+                             int flags, int alive)
+{
+  extern int init_thinkers_count;
+  thinker_t *th, *start_th;
+
+  // If the thinkers have been wiped, addresses are invalid
+  if (*last_count != init_thinkers_count)
+  {
+    *last_count = init_thinkers_count;
+    *last_mobj = NULL;
+  }
+
+  if (*last_mobj)
+    th = &(*last_mobj)->thinker;
+  else
+    th = &thinkercap;
+
+  start_th = th;
+
+  do
+  {
+    th = th->next;
+    if (th->function.p1 == (actionf_p1)P_MobjThinker)
+    {
+      mobj_t *mobj;
+
+      mobj = (mobj_t *) th;
+
+      if ((!alive || mobj->health > 0) && mobj->flags & flags)
+      {
+        followplayer = false;
+        AM_SetMapCenter(mobj->x, mobj->y);
+        P_SetTarget(last_mobj, mobj);
+        break;
+      }
+    }
+  } while (th != start_th);
+}
+
+static void cheat_reveal_kill()
+{
+  if (automapactive)
+  {
+    static int last_count;
+    static mobj_t *last_mobj;
+
+    cheat_cycle_mobj(&last_mobj, &last_count, MF_COUNTKILL, true);
+  }
+}
+
+static void cheat_reveal_item()
+{
+  if (automapactive)
+  {
+    static int last_count;
+    static mobj_t *last_mobj;
+
+    cheat_cycle_mobj(&last_mobj, &last_count, MF_COUNTITEM, false);
+  }
+}
+
 // killough 2/7/98: HOM autodetection
 static void cheat_hom()
 {
   extern int autodetect_hom;           // Ty 03/27/98 - *not* externalized
-  plyr->message = (autodetect_hom = !autodetect_hom) ? "HOM Detection On" :
-    "HOM Detection Off";
+  displaymsg((autodetect_hom = !autodetect_hom) ? "HOM Detection On" :
+    "HOM Detection Off");
 }
 
 // killough 3/6/98: -fast parameter toggle
 static void cheat_fast()
 {
-  plyr->message = (fastparm = !fastparm) ? "Fast Monsters On" : 
-    "Fast Monsters Off";  // Ty 03/27/98 - *not* externalized
+  displaymsg((fastparm = !fastparm) ? "Fast Monsters On" : 
+    "Fast Monsters Off");  // Ty 03/27/98 - *not* externalized
   G_SetFastParms(fastparm); // killough 4/10/98: set -fast parameter correctly
 }
 
 // killough 2/16/98: keycard/skullkey cheat functions
 static void cheat_key()
 {
-  plyr->message = "Red, Yellow, Blue";  // Ty 03/27/98 - *not* externalized
+  displaymsg("Red, Yellow, Blue");  // Ty 03/27/98 - *not* externalized
 }
 
 static void cheat_keyx()
 {
-  plyr->message = "Card, Skull";        // Ty 03/27/98 - *not* externalized
+  displaymsg("Card, Skull");        // Ty 03/27/98 - *not* externalized
 }
 
 static void cheat_keyxx(int key)
 {
-  plyr->message = (plyr->cards[key] = !plyr->cards[key]) ? 
-    "Key Added" : "Key Removed";  // Ty 03/27/98 - *not* externalized
+  displaymsg((plyr->cards[key] = !plyr->cards[key]) ? 
+    "Key Added" : "Key Removed");  // Ty 03/27/98 - *not* externalized
 }
 
 // killough 2/16/98: generalized weapon cheats
 
 static void cheat_weap()
 {                                   // Ty 03/27/98 - *not* externalized
-  plyr->message = have_ssg ? // killough 2/28/98
-    "Weapon number 1-9" : "Weapon number 1-8";
+  displaymsg(have_ssg ? // killough 2/28/98
+    "Weapon number 1-9" : "Weapon number 1-8");
 }
 
 static void cheat_weapx(char *buf)
@@ -907,10 +1092,10 @@ static void cheat_weapx(char *buf)
     if (w >= 0 && w < NUMWEAPONS)
     {
       if ((plyr->weaponowned[w] = !plyr->weaponowned[w]))
-        plyr->message = "Weapon Added";  // Ty 03/27/98 - *not* externalized
+        displaymsg("Weapon Added");  // Ty 03/27/98 - *not* externalized
       else 
         {
-          plyr->message = "Weapon Removed"; // Ty 03/27/98 - *not* externalized
+          displaymsg("Weapon Removed"); // Ty 03/27/98 - *not* externalized
           if (w==plyr->readyweapon)         // maybe switch if weapon removed
             plyr->pendingweapon = P_SwitchWeapon(plyr);
         }
@@ -920,7 +1105,7 @@ static void cheat_weapx(char *buf)
 // killough 2/16/98: generalized ammo cheats
 static void cheat_ammo()
 {
-  plyr->message = "Ammo 1-4, Backpack";  // Ty 03/27/98 - *not* externalized
+  displaymsg("Ammo 1-4, Backpack");  // Ty 03/27/98 - *not* externalized
 }
 
 static void cheat_ammox(char *buf)
@@ -928,10 +1113,10 @@ static void cheat_ammox(char *buf)
   int a = *buf - '1';
   if (*buf == 'b')  // Ty 03/27/98 - strings *not* externalized
     if ((plyr->backpack = !plyr->backpack))
-      for (plyr->message = "Backpack Added",   a=0 ; a<NUMAMMO ; a++)
+      for (displaymsg("Backpack Added"),   a=0 ; a<NUMAMMO ; a++)
         plyr->maxammo[a] <<= 1;
     else
-      for (plyr->message = "Backpack Removed", a=0 ; a<NUMAMMO ; a++)
+      for (displaymsg("Backpack Removed"), a=0 ; a<NUMAMMO ; a++)
         {
           if (plyr->ammo[a] > (plyr->maxammo[a] >>= 1))
             plyr->ammo[a] = plyr->maxammo[a];
@@ -940,28 +1125,33 @@ static void cheat_ammox(char *buf)
     if (a>=0 && a<NUMAMMO)  // Ty 03/27/98 - *not* externalized
       { // killough 5/5/98: switch plasma and rockets for now -- KLUDGE 
         a = a==am_cell ? am_misl : a==am_misl ? am_cell : a;  // HACK
-        plyr->message = (plyr->ammo[a] = !plyr->ammo[a]) ? 
-          plyr->ammo[a] = plyr->maxammo[a], "Ammo Added" : "Ammo Removed";
+        if ((plyr->ammo[a] = !plyr->ammo[a]))
+        {
+          plyr->ammo[a] = plyr->maxammo[a];
+          displaymsg("Ammo Added");
+        }
+        else
+          displaymsg("Ammo Removed");
       }
 }
 
 static void cheat_smart()
 {
-  plyr->message = (monsters_remember = !monsters_remember) ? 
-    "Smart Monsters Enabled" : "Smart Monsters Disabled";
+  displaymsg((monsters_remember = !monsters_remember) ? 
+    "Smart Monsters Enabled" : "Smart Monsters Disabled");
 }
 
 static void cheat_pitch()
 {
-  plyr->message=(pitched_sounds = !pitched_sounds) ? "Pitch Effects Enabled" :
-    "Pitch Effects Disabled";
+  displaymsg((pitched_sounds = !pitched_sounds) ? "Pitch Effects Enabled" :
+    "Pitch Effects Disabled");
 }
 
 static void cheat_nuke()
 {
   extern int disable_nuke;
-  plyr->message = (disable_nuke = !disable_nuke) ? "Nukage Disabled" :
-    "Nukage Enabled";
+  displaymsg((disable_nuke = !disable_nuke) ? "Nukage Disabled" :
+    "Nukage Enabled");
 }
 
 static void cheat_rate()
@@ -976,6 +1166,16 @@ static void cheat_rate()
 // scrambling and to use a more general table-driven approach.
 //-----------------------------------------------------------------------------
 
+static boolean M_CheatAllowed(cheat_when_t when)
+{
+  return
+    !(when & not_dm   && deathmatch && !demoplayback) &&
+    !(when & not_coop && netgame && !deathmatch) &&
+    !(when & not_demo && (demorecording || demoplayback)) &&
+    !(when & not_menu && menuactive) &&
+    !(when & beta_only && !beta_emulation);
+}
+
 #define CHEAT_ARGS_MAX 8  /* Maximum number of args at end of cheats */
 
 boolean M_FindCheats(int key)
@@ -1031,11 +1231,7 @@ boolean M_FindCheats(int key)
 
   for (matchedbefore = ret = i = 0; cheat[i].cheat; i++)
     if ((sr & cheat[i].mask) == cheat[i].code &&  // if match found & allowed
-        !(cheat[i].when & not_dm   && deathmatch && !demoplayback) &&
-        !(cheat[i].when & not_coop && netgame && !deathmatch) &&
-        !(cheat[i].when & not_demo && (demorecording || demoplayback)) &&
-        !(cheat[i].when & not_menu && menuactive) &&
-        !(cheat[i].when & beta_only && !beta_emulation) &&
+        M_CheatAllowed(cheat[i].when) &&
         !(cheat[i].when & not_deh  && cheat[i].deh_modified))
     {
       if (cheat[i].arg < 0)               // if additional args are required
@@ -1055,6 +1251,54 @@ boolean M_FindCheats(int key)
   return ret;
 }
 
+static const struct {
+  int input;
+  const cheat_when_t when;
+  const cheatf_t func;
+  const int arg;
+} cheat_input[] = {
+  { input_iddqd,     not_net|not_demo, {cheat_god},      0 },
+  { input_idkfa,     not_net|not_demo, {cheat_kfa},      0 },
+  { input_idfa,      not_net|not_demo, {cheat_fa},       0 },
+  { input_idclip,    not_net|not_demo, {cheat_noclip},   0 },
+  { input_idbeholdh, not_net|not_demo, {cheat_health},   0 },
+  { input_idbeholdm, not_net|not_demo, {cheat_megaarmour}, 0 },
+  { input_idbeholdv, not_net|not_demo, {cheat_pw},       pw_invulnerability },
+  { input_idbeholds, not_net|not_demo, {cheat_pw},       pw_strength },
+  { input_idbeholdi, not_net|not_demo, {cheat_pw},       pw_invisibility },
+  { input_idbeholdr, not_net|not_demo, {cheat_pw},       pw_ironfeet },
+  { input_idbeholdl, not_dm,           {cheat_pw},       pw_infrared },
+  { input_idrate,    always,           {cheat_rate},     0 },
+  { input_iddt,      not_dm,           {cheat_ddt},      0 },
+  { input_notarget,  not_net|not_demo, {cheat_notarget}, 0 },
+  { input_freeze,    not_net|not_demo, {cheat_freeze},   0 },
+  { input_avj,       not_net|not_demo, {cheat_avj},      0 },
+};
+
+boolean M_CheatResponder(event_t *ev)
+{
+  int i;
+
+  if (strictmode)
+    return false;
+
+  if (ev->type == ev_keydown && M_FindCheats(ev->data1))
+    return true;
+
+  for (i = 0; i < arrlen(cheat_input); ++i)
+  {
+    if (M_InputActivated(cheat_input[i].input))
+    {
+      if (M_CheatAllowed(cheat_input[i].when))
+        cheat_input[i].func.i(cheat_input[i].arg);
+
+      return true;
+    }
+  }
+
+  return false;
+}
+
 //----------------------------------------------------------------------------
 //
 // $Log: m_cheat.c,v $
