@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*-
-//-----------------------------------------------------------------------------
-//
-// $Id: p_inter.c,v 1.10 1998/05/03 23:09:29 killough Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 // DESCRIPTION:
 //      Handling interactions (i.e., collisions).
 //
@@ -62,6 +53,7 @@ int idkfa_armor = 200;
 int idkfa_armor_class = 2;
 
 int bfgcells = 40;      // used in p_pspr.c
+int deh_species_infighting = 0;
 // Ty 03/07/98 - end deh externals
 
 // a weapon is found with two clip loads,
@@ -323,13 +315,13 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
     case SPR_ARM1:
       if (!P_GiveArmor (player, green_armor_class))
         return;
-      player->message = s_GOTARMOR; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTARMOR); // Ty 03/22/98 - externalized
       break;
 
     case SPR_ARM2:
       if (!P_GiveArmor (player, blue_armor_class))
         return;
-      player->message = s_GOTMEGA; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTMEGA); // Ty 03/22/98 - externalized
       break;
 
       // bonus items
@@ -337,7 +329,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
 
       if (beta_emulation)
 	{   // killough 7/11/98: beta version items did not have any effect
-	  player->message = "You pick up a demonic dagger.";
+	  pickupmsg(player, "You pick up a demonic dagger.");
 	  break;
 	}
 
@@ -345,14 +337,14 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
       if (player->health > (maxhealth * 2))
         player->health = (maxhealth * 2);
       player->mo->health = player->health;
-      player->message = s_GOTHTHBONUS; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTHTHBONUS); // Ty 03/22/98 - externalized
       break;
 
     case SPR_BON2:
 
       if (beta_emulation)
 	{ // killough 7/11/98: beta version items did not have any effect
-	  player->message = "You pick up a skullchest.";
+	  pickupmsg(player, "You pick up a skullchest.");
 	  break;
 	}
 
@@ -361,15 +353,15 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
         player->armorpoints = max_armor;
       if (!player->armortype)
         player->armortype = green_armor_class;
-      player->message = s_GOTARMBONUS; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTARMBONUS); // Ty 03/22/98 - externalized
       break;
 
     case SPR_BON3:      // killough 7/11/98: evil sceptre from beta version
-      player->message = "Picked up an evil sceptre";
+      pickupmsg(player, "Picked up an evil sceptre");
       break;
 
     case SPR_BON4:      // killough 7/11/98: unholy bible from beta version
-      player->message = "Picked up an unholy bible";
+      pickupmsg(player, "Picked up an unholy bible");
       break;
 
     case SPR_SOUL:
@@ -377,7 +369,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
       if (player->health > max_soul)
         player->health = max_soul;
       player->mo->health = player->health;
-      player->message = s_GOTSUPER; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTSUPER); // Ty 03/22/98 - externalized
       sound = sfx_getpow;
       break;
 
@@ -387,7 +379,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
       player->health = mega_health;
       player->mo->health = player->health;
       P_GiveArmor (player,blue_armor_class);
-      player->message = s_GOTMSPHERE; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTMSPHERE); // Ty 03/22/98 - externalized
       sound = sfx_getpow;
       break;
 
@@ -395,7 +387,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
         // leave cards for everyone
     case SPR_BKEY:
       if (!player->cards[it_bluecard])
-        player->message = s_GOTBLUECARD; // Ty 03/22/98 - externalized
+        pickupmsg(player, "%s", s_GOTBLUECARD); // Ty 03/22/98 - externalized
       P_GiveCard (player, it_bluecard);
       if (!netgame)
         break;
@@ -403,7 +395,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
 
     case SPR_YKEY:
       if (!player->cards[it_yellowcard])
-        player->message = s_GOTYELWCARD; // Ty 03/22/98 - externalized
+        pickupmsg(player, "%s", s_GOTYELWCARD); // Ty 03/22/98 - externalized
       P_GiveCard (player, it_yellowcard);
       if (!netgame)
         break;
@@ -411,7 +403,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
 
     case SPR_RKEY:
       if (!player->cards[it_redcard])
-        player->message = s_GOTREDCARD; // Ty 03/22/98 - externalized
+        pickupmsg(player, "%s", s_GOTREDCARD); // Ty 03/22/98 - externalized
       P_GiveCard (player, it_redcard);
       if (!netgame)
         break;
@@ -419,7 +411,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
 
     case SPR_BSKU:
       if (!player->cards[it_blueskull])
-        player->message = s_GOTBLUESKUL; // Ty 03/22/98 - externalized
+        pickupmsg(player, "%s", s_GOTBLUESKUL); // Ty 03/22/98 - externalized
       P_GiveCard (player, it_blueskull);
       if (!netgame)
         break;
@@ -427,7 +419,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
 
     case SPR_YSKU:
       if (!player->cards[it_yellowskull])
-        player->message = s_GOTYELWSKUL; // Ty 03/22/98 - externalized
+        pickupmsg(player, "%s", s_GOTYELWSKUL); // Ty 03/22/98 - externalized
       P_GiveCard (player, it_yellowskull);
       if (!netgame)
         break;
@@ -435,7 +427,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
 
     case SPR_RSKU:
       if (!player->cards[it_redskull])
-        player->message = s_GOTREDSKULL; // Ty 03/22/98 - externalized
+        pickupmsg(player, "%s", s_GOTREDSKULL); // Ty 03/22/98 - externalized
       P_GiveCard (player, it_redskull);
       if (!netgame)
         break;
@@ -445,7 +437,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
     case SPR_STIM:
       if (!P_GiveBody (player, 10))
         return;
-      player->message = s_GOTSTIM; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTSTIM); // Ty 03/22/98 - externalized
       break;
 
     case SPR_MEDI:
@@ -454,9 +446,9 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
 
       // [FG] show "Picked up a Medikit that you really need" message as intended
       if (player->health < 50)
-        player->message = s_GOTMEDINEED; // Ty 03/22/98 - externalized
+        pickupmsg(player, "%s", s_GOTMEDINEED); // Ty 03/22/98 - externalized
       else
-        player->message = s_GOTMEDIKIT; // Ty 03/22/98 - externalized
+        pickupmsg(player, "%s", s_GOTMEDIKIT); // Ty 03/22/98 - externalized
       break;
 
 
@@ -464,14 +456,14 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
     case SPR_PINV:
       if (!P_GivePower (player, pw_invulnerability))
         return;
-      player->message = s_GOTINVUL; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTINVUL); // Ty 03/22/98 - externalized
       sound = sfx_getpow;
       break;
 
     case SPR_PSTR:
       if (!P_GivePower (player, pw_strength))
         return;
-      player->message = s_GOTBERSERK; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTBERSERK); // Ty 03/22/98 - externalized
       if (player->readyweapon != wp_fist)
 	if (!beta_emulation // killough 10/98: don't switch as much in -beta
 	    || player->readyweapon == wp_pistol)
@@ -482,7 +474,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
     case SPR_PINS:
       if (!P_GivePower (player, pw_invisibility))
         return;
-      player->message = s_GOTINVIS; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTINVIS); // Ty 03/22/98 - externalized
       sound = sfx_getpow;
       break;
 
@@ -493,14 +485,14 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
       if (beta_emulation)  // killough 7/19/98: beta rad suit did not wear off
 	player->powers[pw_ironfeet] = -1;
 
-      player->message = s_GOTSUIT; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTSUIT); // Ty 03/22/98 - externalized
       sound = sfx_getpow;
       break;
 
     case SPR_PMAP:
       if (!P_GivePower (player, pw_allmap))
         return;
-      player->message = s_GOTMAP; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTMAP); // Ty 03/22/98 - externalized
       sound = sfx_getpow;
       break;
 
@@ -514,7 +506,7 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
 	player->powers[pw_infrared] = -1;
 
       sound = sfx_getpow;
-      player->message = s_GOTVISOR; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTVISOR); // Ty 03/22/98 - externalized
       break;
 
       // ammo
@@ -529,49 +521,49 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
           if (!P_GiveAmmo (player,am_clip,1))
             return;
         }
-      player->message = s_GOTCLIP; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTCLIP); // Ty 03/22/98 - externalized
       break;
 
     case SPR_AMMO:
       if (!P_GiveAmmo (player, am_clip,5))
         return;
-      player->message = s_GOTCLIPBOX; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTCLIPBOX); // Ty 03/22/98 - externalized
       break;
 
     case SPR_ROCK:
       if (!P_GiveAmmo (player, am_misl,1))
         return;
-      player->message = s_GOTROCKET; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTROCKET); // Ty 03/22/98 - externalized
       break;
 
     case SPR_BROK:
       if (!P_GiveAmmo (player, am_misl,5))
         return;
-      player->message = s_GOTROCKBOX; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTROCKBOX); // Ty 03/22/98 - externalized
       break;
 
     case SPR_CELL:
       if (!P_GiveAmmo (player, am_cell,1))
         return;
-      player->message = s_GOTCELL; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTCELL); // Ty 03/22/98 - externalized
       break;
 
     case SPR_CELP:
       if (!P_GiveAmmo (player, am_cell,5))
         return;
-      player->message = s_GOTCELLBOX; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTCELLBOX); // Ty 03/22/98 - externalized
       break;
 
     case SPR_SHEL:
       if (!P_GiveAmmo (player, am_shell,1))
         return;
-      player->message = s_GOTSHELLS; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTSHELLS); // Ty 03/22/98 - externalized
       break;
 
     case SPR_SBOX:
       if (!P_GiveAmmo (player, am_shell,5))
         return;
-      player->message = s_GOTSHELLBOX; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTSHELLBOX); // Ty 03/22/98 - externalized
       break;
 
     case SPR_BPAK:
@@ -583,59 +575,59 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
         }
       for (i=0 ; i<NUMAMMO ; i++)
         P_GiveAmmo (player, i, 1);
-      player->message = s_GOTBACKPACK; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTBACKPACK); // Ty 03/22/98 - externalized
       break;
 
         // weapons
     case SPR_BFUG:
       if (!P_GiveWeapon (player, wp_bfg, false) )
         return;
-      player->message = 
-	classic_bfg || beta_emulation ? 
-	"You got the BFG2704!  Oh, yes." :   // killough 8/9/98: beta BFG
-	  s_GOTBFG9000; // Ty 03/22/98 - externalized
+      if (classic_bfg || beta_emulation)
+        pickupmsg(player, "You got the BFG2704!  Oh, yes."); // killough 8/9/98: beta BFG
+      else
+        pickupmsg(player, "%s", s_GOTBFG9000); // Ty 03/22/98 - externalized
       sound = sfx_wpnup;
       break;
 
     case SPR_MGUN:
       if (!P_GiveWeapon (player, wp_chaingun, special->flags & MF_DROPPED))
         return;
-      player->message = s_GOTCHAINGUN; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTCHAINGUN); // Ty 03/22/98 - externalized
       sound = sfx_wpnup;
       break;
 
     case SPR_CSAW:
       if (!P_GiveWeapon(player, wp_chainsaw, false))
         return;
-      player->message = s_GOTCHAINSAW; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTCHAINSAW); // Ty 03/22/98 - externalized
       sound = sfx_wpnup;
       break;
 
     case SPR_LAUN:
       if (!P_GiveWeapon (player, wp_missile, false) )
         return;
-      player->message = s_GOTLAUNCHER; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTLAUNCHER); // Ty 03/22/98 - externalized
       sound = sfx_wpnup;
       break;
 
     case SPR_PLAS:
       if (!P_GiveWeapon(player, wp_plasma, false))
         return;
-      player->message = s_GOTPLASMA; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTPLASMA); // Ty 03/22/98 - externalized
       sound = sfx_wpnup;
       break;
 
     case SPR_SHOT:
       if (!P_GiveWeapon(player, wp_shotgun, special->flags & MF_DROPPED))
         return;
-      player->message = s_GOTSHOTGUN; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTSHOTGUN); // Ty 03/22/98 - externalized
       sound = sfx_wpnup;
       break;
 
     case SPR_SGN2:
       if (!P_GiveWeapon(player, wp_supershotgun, special->flags & MF_DROPPED))
         return;
-      player->message = s_GOTSHOTGUN2; // Ty 03/22/98 - externalized
+      pickupmsg(player, "%s", s_GOTSHOTGUN2); // Ty 03/22/98 - externalized
       sound = sfx_wpnup;
       break;
 
@@ -752,7 +744,10 @@ static void P_KillMobj(mobj_t *source, mobj_t *target)
   // [crispy] randomly flip corpse, blood and death animation sprites
   if (target->flags2 & MF2_FLIPPABLE)
   {
-    target->health = (target->health & (int)~1) - (Woof_Random() & 1);
+    if (Woof_Random() & 1)
+      target->intflags |= MIF_FLIP;
+    else
+      target->intflags &= ~MIF_FLIP;
   }
 
   if (target->tics < 1)
