@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*-
-//-----------------------------------------------------------------------------
-//
-// $Id: p_map.c,v 1.35 1998/05/12 12:47:16 phares Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 // DESCRIPTION:
 //  Movement, collision handling.
 //  Shooting and aiming.
@@ -573,9 +564,13 @@ static boolean PIT_CheckThing(mobj_t *thing) // killough 3/26/98: make static
   if (tmthing->flags & MF_MISSILE || (tmthing->flags & MF_BOUNCES &&
 				      !(tmthing->flags & MF_SOLID)))
     {
+      // [crispy] mobj or actual sprite height
+      const fixed_t thingheight = (CRITICAL(direct_vertical_aiming) && tmthing->target && tmthing->target->player) ?
+        thing->info->actualheight : thing->height;
+
       // see if it went over / under
 
-      if (tmthing->z > thing->z + thing->height)
+      if (tmthing->z > thing->z + thingheight)
 	return true;    // overhead
 
       if (tmthing->z+tmthing->height < thing->z)
@@ -586,7 +581,8 @@ static boolean PIT_CheckThing(mobj_t *thing) // killough 3/26/98: make static
 	if (thing == tmthing->target)
 	  return true;                // Don't hit same species as originator.
 	else
-	  if (thing->type != MT_PLAYER)	// Explode, but do no damage.
+	  // Dehacked support - monsters infight
+	  if (thing->type != MT_PLAYER && !deh_species_infighting) // Explode, but do no damage.
 	    return false;	        // Let players missile other players.
       }
       
@@ -1573,6 +1569,29 @@ static boolean PTR_ShootTraverse(intercept_t *in)
 	      return false;
 	}
 
+      // [crispy] check if the pullet puff's z-coordinate is below or above
+      // its spawning sector's floor or ceiling, respectively, and move its
+      // coordinates to the point where the trajectory hits the plane
+      if (CRITICAL(direct_vertical_aiming) && aimslope)
+      {
+        const int lineside = P_PointOnLineSide(x, y, li);
+        int side;
+
+        if ((side = li->sidenum[lineside]) != NO_INDEX)
+        {
+          const sector_t *const sector = sides[side].sector;
+
+          if (z < sector->floorheight ||
+             (z > sector->ceilingheight && sector->ceilingpic != skyflatnum))
+          {
+            z = BETWEEN(sector->floorheight, sector->ceilingheight, z);
+            frac = FixedDiv(z - shootz, FixedMul(aimslope, attackrange));
+            x = trace.x + FixedMul (trace.dx, frac);
+            y = trace.y + FixedMul (trace.dy, frac);
+          }
+        }
+      }
+
       // Spawn bullet puffs.
 
       P_SpawnPuff (x,y,z);
@@ -1594,7 +1613,12 @@ static boolean PTR_ShootTraverse(intercept_t *in)
   // check angles to see if the thing can be aimed at
 
   dist = FixedMul (attackrange, in->frac);
-  thingtopslope = FixedDiv (th->z+th->height - shootz , dist);
+  {
+  // [crispy] mobj or actual sprite height
+  const fixed_t thingheight = (CRITICAL(direct_vertical_aiming) && shootthing->player) ?
+    th->info->actualheight : th->height;
+  thingtopslope = FixedDiv (th->z+thingheight - shootz , dist);
+  }
 
   if (thingtopslope < aimslope)
     return true;  // shot over the thing
@@ -1648,8 +1672,15 @@ fixed_t P_AimLineAttack(mobj_t *t1,angle_t angle,fixed_t distance,int mask)
 
   // can't shoot outside view angles
 
+  if (t1->player && CRITICAL(direct_vertical_aiming))
+  {
+    bottomslope = (topslope = PLAYER_SLOPE(t1->player) + 1) - 2;
+  }
+  else
+  {
   topslope = 100*FRACUNIT/160;
   bottomslope = -100*FRACUNIT/160;
+  }
 
   attackrange = distance;
   linetarget = NULL;
@@ -1847,7 +1878,7 @@ boolean PIT_RadiusAttack(mobj_t *thing)
 
 void P_RadiusAttack(mobj_t *spot, mobj_t *source, int damage, int distance)
 {
-  fixed_t dist = (damage+MAXRADIUS)<<FRACBITS;
+  fixed_t dist = (distance+MAXRADIUS)<<FRACBITS;
   int yh = (spot->y + dist - bmaporgy)>>MAPBLOCKSHIFT;
   int yl = (spot->y - dist - bmaporgy)>>MAPBLOCKSHIFT;
   int xh = (spot->x + dist - bmaporgx)>>MAPBLOCKSHIFT;
