@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*- 
-//-----------------------------------------------------------------------------
-//
-// $Id: p_mobj.h,v 1.10 1998/05/03 23:45:09 killough Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 // DESCRIPTION:
 //      Map Objects, MObj, definition and handling.
 //
@@ -240,6 +231,8 @@ enum {
   MIF_LINEDONE = 4,     // Object has activated W1 or S1 linedef via DEH frame
   // mbf21
   MIF_SCROLLING = 8,    // Object is affected by scroller / pusher / puller
+  // cosmetic
+  MIF_FLIP = 16,
 };
 
 // Map Object definition.
@@ -423,6 +416,15 @@ extern int iquetail;
 // [FG] colored blood and gibs
 extern boolean colored_blood;
 
+// The value `160` is used for the denominator because in Vanilla Doom
+// the horizontal FOV is 90° and spans 320 px, so the height of the
+// resulting triangle is 160.
+// Heretic/Hexen used value `173` presumably because Raven assumed an
+// equilateral triangle (i.e. 60°) for the vertical FOV which spans
+// 200 px, so the height of that triangle would be 100*sqrt(3) = 173.
+#define PLAYER_SLOPE(a)	((((a)->lookdir / MLOOKUNIT) << FRACBITS) / 160)
+extern boolean direct_vertical_aiming;
+
 mobj_t *P_SubstNullMobj(mobj_t *mobj);
 void    P_RespawnSpecials(void);
 mobj_t  *P_SpawnMobj(fixed_t x, fixed_t y, fixed_t z, mobjtype_t type);
