@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*-
-//-----------------------------------------------------------------------------
-//
-// $Id: p_pspr.c,v 1.13 1998/05/07 00:53:36 killough Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 // DESCRIPTION:
 //      Weapon sprite animation, weapon objects.
 //      Action functions for weapons.
@@ -38,6 +29,7 @@
 #include "sounds.h"
 #include "d_event.h"
 #include "p_tick.h"
+#include "i_video.h" // uncapped
 
 #include "p_action.h"
 
@@ -161,6 +153,7 @@ static void P_BringUpWeapon(player_t *player)
     WEAPONBOTTOM+FRACUNIT*2 : WEAPONBOTTOM;
 
   P_SetPsprite(player, ps_weapon, newstate);
+  player->switching = weapswitch_raising;
 }
 
 // The first set is where the weapon preferences from             // killough,
@@ -442,6 +435,20 @@ static void P_FireWeapon(player_t *player)
 void P_DropWeapon(player_t *player)
 {
   P_SetPsprite(player, ps_weapon, weaponinfo[player->readyweapon].downstate);
+  player->switching = weapswitch_lowering;
+}
+
+//
+// P_ApplyBobbing
+// Bob the weapon based on movement speed.
+//
+
+static void P_ApplyBobbing(int *sx, int *sy, fixed_t bob)
+{
+  int angle = (128*leveltime) & FINEMASK;
+  *sx = FRACUNIT + FixedMul(bob, finecosine[angle]);
+  angle &= FINEANGLES/2-1;
+  *sy = WEAPONTOP + FixedMul(bob, finesine[angle]);
 }
 
 //
@@ -470,8 +477,11 @@ void A_WeaponReady(player_t *player, pspdef_t *psp)
       // change weapon (pending weapon should already be validated)
       statenum_t newstate = weaponinfo[player->readyweapon].downstate;
       P_SetPsprite(player, ps_weapon, newstate);
+      player->switching = weapswitch_lowering;
       return;
     }
+  else
+    player->switching = weapswitch_none;
 
   // check for fire
   //  the missile launcher and bfg do not auto fire
@@ -488,13 +498,7 @@ void A_WeaponReady(player_t *player, pspdef_t *psp)
   else
     player->attackdown = false;
 
-  // bob the weapon based on movement speed
-  {
-    int angle = (128*leveltime) & FINEMASK;
-    psp->sx = FRACUNIT + FixedMul(player->bob, finecosine[angle]);
-    angle &= FINEANGLES/2-1;
-    psp->sy = WEAPONTOP + FixedMul(player->bob, finesine[angle]);
-  }
+  P_ApplyBobbing(&psp->sx, &psp->sy, player->bob);
 }
 
 //
@@ -782,6 +786,11 @@ void A_FireOldBFG(player_t *player, pspdef_t *psp)
 	  // killough 8/2/98: make autoaiming prefer enemies
 	  int mask = MF_FRIEND;
 	  fixed_t slope;
+	  if (CRITICAL(direct_vertical_aiming))
+	  {
+	    slope = PLAYER_SLOPE(mo->player);
+	  }
+	  else
 	  do
 	    {
 	      slope = P_AimLineAttack(mo, an, 16*64*FRACUNIT, mask);
@@ -847,6 +856,11 @@ static void P_BulletSlope(mobj_t *mo)
   // killough 8/2/98: make autoaiming prefer enemies
   int mask = demo_version < 203 ? 0 : MF_FRIEND;
 
+  if (CRITICAL(direct_vertical_aiming))
+  {
+    bulletslope = PLAYER_SLOPE(mo->player);
+  }
+  else
   do
     {
       bulletslope = P_AimLineAttack(mo, an, 16*64*FRACUNIT, mask);
@@ -1066,6 +1080,7 @@ void P_SetupPsprites(player_t *player)
 void P_MovePsprites(player_t *player)
 {
   pspdef_t *psp = player->psprites;
+  const int center_weapon_strict = STRICTMODE(center_weapon);
   int i;
 
   // a null state means not active
@@ -1084,45 +1099,42 @@ void P_MovePsprites(player_t *player)
   psp->sx2 = psp->sx;
   psp->sy2 = psp->sy;
 
-  if (psp->state && !cosmetic_bobbing)
+  if (psp->state)
   {
-    static fixed_t last_sy = 32 * FRACUNIT;
+    if (!cosmetic_bobbing)
+    {
+      static fixed_t last_sy = WEAPONTOP;
 
-    psp->sx2 = FRACUNIT;
-
-    if (psp->state->action.p2 != (actionf_p2)A_Lower &&
-        psp->state->action.p2 != (actionf_p2)A_Raise)
-    {
-      last_sy = psp->sy2;
-      psp->sy2 = 32 * FRACUNIT;
-    }
-    else if (psp->state->action.p2 == (actionf_p2)A_Lower)
-    {
-      // We want to move smoothly from where we were
-      psp->sy2 -= (last_sy - 32 * FRACUNIT);
-    }
-  }
-  else if (psp->state && (cosmetic_bobbing == BOBBING_75 || center_weapon))
-  {
-    // [FG] don't center during lowering and raising states
-    if (psp->state->misc1 ||
-        psp->state->action.p2 == (actionf_p2)A_Lower ||
-        psp->state->action.p2 == (actionf_p2)A_Raise)
-    {
-    }
-    // [FG] not attacking means idle
-    else if (!player->attackdown || center_weapon == WEAPON_BOBBING)
-    {
-      int angle = (128*leveltime) & FINEMASK;
-      psp->sx2 = FRACUNIT + FixedMul(player->bob2, finecosine[angle]);
-      angle &= FINEANGLES/2-1;
-      psp->sy2 = WEAPONTOP + FixedMul(player->bob2, finesine[angle]);
-    }
-    // [FG] center the weapon sprite horizontally and push up vertically
-    else if (center_weapon == WEAPON_CENTERED)
-    {
       psp->sx2 = FRACUNIT;
-      psp->sy2 = WEAPONTOP;
+
+      if (!psp->state->misc1 && !player->switching)
+      {
+        last_sy = psp->sy2;
+        psp->sy2 = WEAPONTOP;
+      }
+      else if (player->switching == weapswitch_lowering)
+      {
+        // We want to move smoothly from where we were
+        psp->sy2 -= (last_sy - WEAPONTOP);
+      }
+    }
+    else if (cosmetic_bobbing == BOBBING_75 || center_weapon_strict || uncapped)
+    {
+      // [FG] don't center during lowering and raising states
+      if (psp->state->misc1 || player->switching)
+      {
+      }
+      // [FG] not attacking means idle
+      else if (!player->attackdown || center_weapon_strict == WEAPON_BOBBING)
+      {
+        P_ApplyBobbing(&psp->sx2, &psp->sy2, player->bob2);
+      }
+      // [FG] center the weapon sprite horizontally and push up vertically
+      else if (center_weapon_strict == WEAPON_CENTERED)
+      {
+        psp->sx2 = FRACUNIT;
+        psp->sy2 = WEAPONTOP;
+      }
     }
   }
 
