@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*-
-//-----------------------------------------------------------------------------
-//
-// $Id: r_bsp.c,v 1.17 1998/05/03 22:47:33 killough Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 // DESCRIPTION:
 //      BSP traversal, handling of LineSegs for rendering.
 //
@@ -279,19 +270,30 @@ sector_t *R_FakeFlat(sector_t *sec, sector_t *tempsec,
 // [AM] Interpolate the passed sector, if prudent.
 static void R_MaybeInterpolateSector(sector_t* sector)
 {
-    if (uncapped &&
-        // Only if we moved the sector last tic.
-        sector->oldgametic == gametic - 1)
+    if (uncapped)
     {
         // Interpolate between current and last floor/ceiling position.
-        if (sector->floordata && sector->floorheight != sector->oldfloorheight)
+        if (sector->floordata && sector->floorheight != sector->oldfloorheight &&
+            // Only if we moved the sector last tic.
+            sector->oldfloorgametic == gametic - 1)
+        {
             sector->interpfloorheight = sector->oldfloorheight + FixedMul(sector->floorheight - sector->oldfloorheight, fractionaltic);
+        }
         else
+        {
             sector->interpfloorheight = sector->floorheight;
-        if (sector->ceilingdata && sector->ceilingheight != sector->oldceilingheight)
+        }
+
+        if (sector->ceilingdata && sector->ceilingheight != sector->oldceilingheight &&
+            // Only if we moved the sector last tic.
+            sector->oldceilgametic == gametic - 1)
+        {
             sector->interpceilingheight = sector->oldceilingheight + FixedMul(sector->ceilingheight - sector->oldceilingheight, fractionaltic);
+        }
         else
+        {
             sector->interpceilingheight = sector->ceilingheight;
+        }
     }
     else
     {
@@ -376,16 +378,16 @@ static void R_AddLine (seg_t *line)
   if (!backsector)
     goto clipsolid;
 
-  // killough 3/8/98, 4/4/98: hack for invisible ceilings / deep water
-  backsector = R_FakeFlat(backsector, &tempsec, NULL, NULL, true);
-
-  doorclosed = 0;       // killough 4/16/98
-
   // [AM] Interpolate sector movement before
   //      running clipping tests.  Frontsector
   //      should already be interpolated.
   R_MaybeInterpolateSector(backsector);
 
+  // killough 3/8/98, 4/4/98: hack for invisible ceilings / deep water
+  backsector = R_FakeFlat(backsector, &tempsec, NULL, NULL, true);
+
+  doorclosed = 0;       // killough 4/16/98
+
   // Closed door.
 
   if (backsector->interpceilingheight <= frontsector->interpfloorheight
