@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*-
-//-----------------------------------------------------------------------------
-//
-// $Id: r_data.c,v 1.23 1998/05/23 08:05:57 killough Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 // DESCRIPTION:
 //      Preparation of data for rendering,
 //      generation of lookups, caching, retrieval by name.
@@ -122,6 +113,7 @@ unsigned  **texturecolumnofs2;
 byte      **texturecomposite;
 byte      **texturecomposite2;
 int       *flattranslation;             // for global animation
+int       *flatterrain;
 int       *texturetranslation;
 const byte **texturebrightmap; // [crispy] brightmaps
 
@@ -342,17 +334,6 @@ static void R_GenerateLookup(int texnum, int *const errors)
       int x, x1 = patch++->originx, x2 = x1 + SHORT(realpatch->width);
       const int *cofs = realpatch->columnofs - x1;
       
-	// [crispy] detect patches in PNG format... and fail
-	{
-		const unsigned char *magic = (const unsigned char *) realpatch;
-
-		if (magic[0] == 0x89 &&
-		    magic[1] == 'P' && magic[2] == 'N' && magic[3] == 'G')
-		{
-			I_Error("Patch in PNG format detected: %.8s", lumpinfo[pat].name);
-		}
-	}
-
       if (x2 > texture->width)
 	x2 = texture->width;
       if (x1 < 0)
@@ -391,14 +372,6 @@ static void R_GenerateLookup(int texnum, int *const errors)
 	  int x, x1 = patch++->originx, x2 = x1 + SHORT(realpatch->width);
 	  const int *cofs = realpatch->columnofs - x1;
 	  
-	  if (!R_IsPatchLump(pat))
-	  {
-	    fprintf(stderr, "\nR_GenerateLookup: Texture %.8s"
-	                    " patch num %d (%.8s) is not valid",
-	                    texture->name, i, lumpinfo[pat].name);
-	    continue;
-	  }
-
 	  if (x2 > texture->width)
 	    x2 = texture->width;
 	  if (x1 < 0)
@@ -582,6 +555,13 @@ void R_InitTextures (void)
           if (patchlookup[i] == -1 && devparm)	    // killough 8/8/98
             printf("\nWarning: patch %.8s, index %d does not exist",name,i);
         }
+
+      if (patchlookup[i] != -1 && !R_IsPatchLump(patchlookup[i]))
+        {
+          fprintf(stderr, "\nR_InitTextures: patch %.8s, index %d is invalid", name, i);
+          patchlookup[i] = (W_CheckNumForName)("TNT1A0", ns_sprites);
+        }
+
     }
   Z_Free(names);
 
@@ -780,8 +760,14 @@ void R_InitFlats(void)
   flattranslation =
     Z_Malloc((numflats+1)*sizeof(*flattranslation), PU_STATIC, 0);
 
+  flatterrain =
+    Z_Malloc((numflats+1)*sizeof(*flatterrain), PU_STATIC, 0);
+
   for (i=0 ; i<numflats ; i++)
+  {
     flattranslation[i] = i;
+    flatterrain[i] = 0; // terrain_solid
+  }
 }
 
 //
@@ -873,6 +859,11 @@ int tran_filter_pct = 66;       // filter percent
 void R_InitTranMap(int progress)
 {
   int lump = W_CheckNumForName("TRANMAP");
+  //!
+  // @category mod
+  //
+  // Forces a (re-)building of the translucency and color translation tables.
+  //
   int force_rebuild = M_CheckParm("-tranmap");
 
   // If a tranlucency filter map lump is present, use it
@@ -967,7 +958,7 @@ void R_InitTranMap(int progress)
                   }
               }
           }
-          if (cachefp)        // write out the cached translucency map
+          if (cachefp && !force_rebuild) // write out the cached translucency map
             {
               cache.pct = tran_filter_pct;
               memcpy(cache.playpal, playpal, sizeof cache.playpal); // [FG] a palette has 256 colors saved as byte triples
@@ -999,11 +990,10 @@ void R_InitData(void)
 {
   // [crispy] Moved R_InitFlats() to the top, because it sets firstflat/lastflat
   // which are required by R_InitTextures() to prevent flat lumps from being
-  // mistaken as patches and by R_InitBrightmaps() to set brightmaps for flats.
-  // R_InitBrightmaps() comes next, because it sets R_BrightmapForTexName()
-  // to initialize brightmaps depending on gameversion in R_InitTextures().
+  // mistaken as patches and by R_InitFlatBrightmaps() to set brightmaps for
+  // flats.
   R_InitFlats();
-  R_InitBrightmaps();
+  R_InitFlatBrightmaps();
   R_InitTextures();
   R_InitSpriteLumps();
     R_InitTranMap(1);                   // killough 2/21/98, 3/6/98
@@ -1162,7 +1152,7 @@ boolean R_IsPatchLump (const int lump)
 {
   int size;
   int width, height;
-  const patch_t * patch;
+  const patch_t *patch;
   boolean result;
 
   size = W_LumpLength(lump);
@@ -1173,6 +1163,10 @@ boolean R_IsPatchLump (const int lump)
 
   patch = (const patch_t *)W_CacheLumpNum(lump, PU_CACHE);
 
+  // [FG] detect patches in PNG format early
+  if (!memcmp(patch, "\211PNG\r\n\032\n", 8))
+    return false;
+
   width = SHORT(patch->width);
   height = SHORT(patch->height);
 
