@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*- 
-//-----------------------------------------------------------------------------
-//
-// $Id: r_main.c,v 1.13 1998/05/07 00:47:52 killough Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 // DESCRIPTION:
 //      Rendering main loop and setup functions,
 //       utility functions (BSP, geometry, trigonometry).
@@ -38,6 +29,7 @@
 #include "r_sky.h"
 #include "v_video.h"
 #include "st_stuff.h"
+#include "hu_stuff.h"
 
 // Fineangles in the SCREENWIDTH wide window.
 #define FIELDOFVIEW 2048    
@@ -510,11 +502,11 @@ void R_ExecuteSetViewSize (void)
       for (j = 0; j < LOOKDIRS; j++)
       {
         // [crispy] re-generate lookup-table for yslope[] whenever "viewheight" or "hires" change
-        fixed_t dy = abs(((i-viewheight/2-(j-LOOKDIRMIN)*viewblocks/10)<<FRACBITS)+FRACUNIT/2);
+        fixed_t dy = abs(((i-viewheight/2-(j-LOOKDIRMAX)*viewblocks/10)<<FRACBITS)+FRACUNIT/2);
         yslopes[j][i] = FixedDiv(viewwidth_nonwide*(FRACUNIT/2), dy);
       }
     }
-  yslope = yslopes[LOOKDIRMIN];
+  yslope = yslopes[LOOKDIRMAX];
         
   for (i=0 ; i<viewwidth ; i++)
     {
@@ -550,6 +542,8 @@ void R_ExecuteSetViewSize (void)
         }
     }
 
+    HU_disableAllWidgets();
+
     // [crispy] forcefully initialize the status bar backing screen
     ST_refreshBackground(true);
 
@@ -658,8 +652,8 @@ void R_SetupFrame (player_t *player)
     
   if (pitch > LOOKDIRMAX)
     pitch = LOOKDIRMAX;
-  else if (pitch < -LOOKDIRMIN)
-    pitch = -LOOKDIRMIN;
+  else if (pitch < -LOOKDIRMAX)
+    pitch = -LOOKDIRMAX;
 
   // apply new yslope[] whenever "lookdir", "viewheight" or "hires" change
   tempCentery = viewheight/2 + pitch * viewblocks / 10;
@@ -667,7 +661,7 @@ void R_SetupFrame (player_t *player)
   {
       centery = tempCentery;
       centeryfrac = centery << FRACBITS;
-      yslope = yslopes[LOOKDIRMIN + pitch];
+      yslope = yslopes[LOOKDIRMAX + pitch];
   }
 
   viewsin = finesine[viewangle>>ANGLETOFINESHIFT];
@@ -715,7 +709,7 @@ int rendered_visplanes, rendered_segs, rendered_vissprites;
 void R_ShowRenderingStats(void)
 {
   extern int fps;
-  doomprintf("Segs %d, Visplanes %d, Sprites %d, FPS %d",
+  displaymsg("Segs %d, Visplanes %d, Sprites %d, FPS %d",
           rendered_segs, rendered_visplanes, rendered_vissprites, fps);
 }
 
