@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*-
-//-----------------------------------------------------------------------------
-//
-// $Id: r_things.c,v 1.22 1998/05/03 22:46:41 killough Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 //
 // DESCRIPTION:
 //  Refresh of things, i.e. objects represented by sprites.
@@ -200,7 +191,12 @@ void R_InitSpriteDefs(char **namelist)
   for (i=0 ; i<num_sprites ; i++)
     {
       const char *spritename = namelist[i];
-      int j = hash[R_SpriteNameHash(spritename) % numentries].index;
+      int j;
+
+      if (!spritename)
+        continue;
+
+      j = hash[R_SpriteNameHash(spritename) % numentries].index;
 
       if (j >= 0)
         {
@@ -402,7 +398,7 @@ void R_DrawVisSprite(vissprite_t *vis, int x1, int x2)
     if (vis->mobjflags2 & MF2_COLOREDBLOOD)
       {
         colfunc = R_DrawTranslatedColumn;
-        dc_translation = (byte *)colrngs[vis->color];
+        dc_translation = (byte *)red2col[vis->color];
       }
   else
     if (vis->mobjflags & MF_TRANSLATION)
@@ -544,7 +540,7 @@ void R_ProjectSprite (mobj_t* thing)
   if (STRICTMODE(flipcorpses) &&
       (thing->flags2 & MF2_FLIPPABLE) &&
       !(thing->flags & MF_SHOOTABLE) &&
-      (thing->health & 1))
+      (thing->intflags & MIF_FLIP))
     {
       flip = !flip;
     }
@@ -680,6 +676,9 @@ void R_AddSprites(sector_t* sec, int lightlevel)
   // Well, now it will be done.
   sec->validcount = validcount;
 
+  if (demo_version <= 202)
+    lightlevel = sec->lightlevel;
+
   lightnum = (lightlevel >> LIGHTSEGSHIFT)+extralight;
 
   if (lightnum < 0)
@@ -815,9 +814,11 @@ void R_DrawPSprite (pspdef_t *psp)
 
     if (lump == oldlump && pspr_interp)
     {
-      int deltax = vis->x2 - vis->x1;
+      int deltax = x2 - vis->x1;
       vis->x1 = oldx1 + FixedMul(vis->x1 - oldx1, fractionaltic);
       vis->x2 = vis->x1 + deltax;
+      if (vis->x2 >= viewwidth)
+        vis->x2 = viewwidth - 1;
       vis->texturemid = oldtexturemid + FixedMul(vis->texturemid - oldtexturemid, fractionaltic);
     }
     else
@@ -832,6 +833,9 @@ void R_DrawPSprite (pspdef_t *psp)
   // [crispy] free look
   vis->texturemid += (centery - viewheight/2) * pspriteiscale;
 
+  if (STRICTMODE(hide_weapon))
+    return;
+
   R_DrawVisSprite(vis, vis->x1, vis->x2);
 }
 
