@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*-
-//-----------------------------------------------------------------------------
-//
-// $Id: st_stuff.c,v 1.46 1998/05/06 16:05:40 jim Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 //
 // DESCRIPTION:
 //      Status bar code.
@@ -35,7 +26,7 @@
 #include "i_video.h"
 #include "w_wad.h"
 #include "st_stuff.h"
-#include "hu_stuff.h" // [FG] hud_displayed, hud_distributed
+#include "hu_stuff.h" // [FG] hud_displayed
 #include "st_lib.h"
 #include "r_main.h"
 #include "am_map.h"
@@ -195,24 +186,9 @@ static player_t *plyr;
 // ST_Start() has just been called
 static boolean st_firsttime;
 
-// used to execute ST_Init() only once
-static int veryfirsttime = 1;
-
 // lump number for PLAYPAL
 static int lu_palette;
 
-// used for timing
-static unsigned int st_clock;
-
-// used for making messages go away
-static int st_msgcounter=0;
-
-// used when in chat
-static st_chatstateenum_t st_chatstate;
-
-// whether in automap or first-person
-static st_stateenum_t st_gamestate;
-
 // whether left-side main status bar is active
 static boolean st_statusbaron;
 
@@ -220,15 +196,6 @@ static boolean st_statusbaron;
 boolean st_crispyhud;
 static boolean st_classicstatusbar;
 
-// whether status bar chat is active
-static boolean st_chat;
-
-// value of st_chat before message popped up
-static boolean st_oldchat;
-
-// whether chat window has the cursor on
-static boolean st_cursoron;
-
 // !deathmatch
 static boolean st_notdeathmatch;
 
@@ -241,6 +208,9 @@ static boolean st_fragson;
 // main bar left
 static patch_t *sbar;
 
+// main bar right, for doom 1.0
+static patch_t *sbarr;
+
 // 0-9, tall numbers
 static patch_t *tallnum[10];
 
@@ -270,6 +240,11 @@ static patch_t *arms[6][2];
 // ready-weapon widget
 static st_number_t w_ready;
 
+// [Alaux]
+int smooth_counts;
+int st_health = 100;
+int st_armor = 0;
+
 //jff 2/16/98 status color change levels
 int ammo_red;      // ammo percent less than which status is red
 int ammo_yellow;   // ammo percent less is yellow more green
@@ -451,6 +426,10 @@ void ST_refreshBackground(boolean force)
       // [crispy] center unity rerelease wide status bar
       V_DrawPatch(st_x, 0, BG, sbar);
 
+      // draw right side of bar if needed (Doom 1.0)
+      if (sbarr)
+        V_DrawPatch(ST_ARMSBGX, 0, BG, sbarr);
+
       if (st_notdeathmatch)
         V_DrawPatch(ST_ARMSBGX, 0, BG, armsbg);
 
@@ -484,21 +463,13 @@ boolean ST_Responder(event_t *ev)
   // Filter automap on/off.
   if (ev->type == ev_keyup && (ev->data1 & 0xffff0000) == AM_MSGHEADER)
     {
-      switch(ev->data1)
-        {
-        case AM_MSGENTERED:
-          st_gamestate = AutomapState;
-          st_firsttime = true;
-          break;
-
-        case AM_MSGEXITED:
-          st_gamestate = FirstPersonState;
-          break;
-        }
+      if (ev->data1 == AM_MSGENTERED)
+      {
+        st_firsttime = true;
+      }
     }
   else  // if a user keypress...
-    if (ev->type == ev_keydown)       // Try cheat responder in m_cheat.c
-      return M_FindCheats(ev->data1); // killough 4/17/98, 5/2/98
+    return M_CheatResponder(ev);       // Try cheat responder in m_cheat.c
   return false;
 }
 
@@ -773,18 +744,47 @@ void ST_updateWidgets(void)
         st_fragscount -= plyr->frags[i];
     }
 
-  // get rid of chat window if up because of message
-  if (!--st_msgcounter)
-    st_chat = st_oldchat;
-
 }
 
+// [Alaux]
+static int SmoothCount(int shownval, int realval)
+{
+  int step = realval - shownval;
+
+  if (!smooth_counts || !step)
+  {
+    return realval;
+  }
+  else
+  {
+    int sign = step / abs(step);
+    step = BETWEEN(1, 7, abs(step) / 20);
+    shownval += (step+1)*sign;
+  
+    if (  (sign > 0 && shownval > realval)
+        ||(sign < 0 && shownval < realval))
+    {
+      shownval = realval;
+    }
+
+    return shownval;
+  }
+}
+
+boolean st_invul;
+
 void ST_Ticker(void)
 {
-  st_clock++;
+  st_health = SmoothCount(st_health, plyr->health);
+  st_armor  = SmoothCount(st_armor, plyr->armorpoints);
+  
   st_randomnumber = M_Random();
   ST_updateWidgets();
   st_oldhealth = plyr->health;
+
+  st_invul = (plyr->powers[pw_invulnerability] > 4*32 ||
+              plyr->powers[pw_invulnerability] & 8) ||
+              plyr->cheats & CF_GODMODE;
 }
 
 static int st_palette = 0;
@@ -865,9 +865,9 @@ void ST_drawWidgets(void)
   int i;
   int maxammo = plyr->maxammo[weaponinfo[w_ready.data].ammo];
 
-  boolean st_invul = (plyr->powers[pw_invulnerability] > 4*32 ||
-                      plyr->powers[pw_invulnerability] & 8) ||
-                      plyr->cheats & CF_GODMODE;
+  // [Alaux] Used to color health and armor counts based on
+  // the real values, only ever relevant when using smooth counts
+  const int health = plyr->health,  armor = plyr->armorpoints;
 
   // clear area
   if (!st_crispyhud && st_statusbaron)
@@ -908,11 +908,11 @@ void ST_drawWidgets(void)
     STlib_updatePercent(&w_health, cr_gray);
   else
   //jff 2/16/98 make color of health depend on amount
-  if (*w_health.n.num<health_red)
+  if (health<health_red)
     STlib_updatePercent(&w_health, cr_red);
-  else if (*w_health.n.num<health_yellow)
+  else if (health<health_yellow)
     STlib_updatePercent(&w_health, cr_gold);
-  else if (*w_health.n.num<=health_green)
+  else if (health<=health_green)
     STlib_updatePercent(&w_health, cr_green);
   else
     STlib_updatePercent(&w_health, cr_blue2); //killough 2/28/98
@@ -935,11 +935,11 @@ void ST_drawWidgets(void)
     STlib_updatePercent(&w_armor, cr_gray);
   else
   //jff 2/16/98 make color of armor depend on amount
-  if (*w_armor.n.num<armor_red)
+  if (armor<armor_red)
     STlib_updatePercent(&w_armor, cr_red);
-  else if (*w_armor.n.num<armor_yellow)
+  else if (armor<armor_yellow)
     STlib_updatePercent(&w_armor, cr_gold);
-  else if (*w_armor.n.num<=armor_green)
+  else if (armor<=armor_green)
     STlib_updatePercent(&w_armor, cr_green);
   else
     STlib_updatePercent(&w_armor, cr_blue2); //killough 2/28/98
@@ -957,25 +957,6 @@ void ST_drawWidgets(void)
 
 }
 
-void ST_doRefresh(void)
-{
-
-  st_firsttime = false;
-
-  // draw status bar background to off-screen buff
-  ST_refreshBackground(false);
-
-  // and refresh all widgets
-  ST_drawWidgets();
-
-}
-
-void ST_diffDraw(void)
-{
-  // update all widgets
-  ST_drawWidgets();
-}
-
 static void ST_MoveHud (void);
 
 void ST_Drawer(boolean fullscreen, boolean refresh)
@@ -992,10 +973,15 @@ void ST_Drawer(boolean fullscreen, boolean refresh)
 
   ST_doPaletteStuff();  // Do red-/gold-shifts from damage/items
 
-  if (st_firsttime)
-    ST_doRefresh();     // If just after ST_Start(), refresh all
-  else
-    ST_diffDraw();      // Otherwise, update as little as possible
+  if (st_firsttime)     // If just after ST_Start(), refresh all
+  {
+    st_firsttime = false;
+
+    // draw status bar background to off-screen buff
+    ST_refreshBackground(false);
+  }
+  
+  ST_drawWidgets();
 }
 
 void ST_loadGraphics(void)
@@ -1048,7 +1034,16 @@ void ST_loadGraphics(void)
     }
 
   // status bar background bits
-  sbar = (patch_t *) W_CacheLumpName("STBAR", PU_STATIC);
+  if (W_CheckNumForName("STBAR") >= 0)
+  {
+    sbar  = (patch_t *) W_CacheLumpName("STBAR", PU_STATIC);
+    sbarr = NULL;
+  }
+  else
+  {
+    sbar  = (patch_t *) W_CacheLumpName("STMBARL", PU_STATIC);
+    sbarr = (patch_t *) W_CacheLumpName("STMBARR", PU_STATIC);
+  }
 
   // face states
   facenum = 0;
@@ -1093,49 +1088,6 @@ void ST_loadData(void)
   ST_loadGraphics();
 }
 
-void ST_unloadGraphics(void)
-{
-  int i;
-
-  // unload the numbers, tall and short
-  for (i=0;i<10;i++)
-    {
-      Z_ChangeTag(tallnum[i], PU_CACHE);
-      Z_ChangeTag(shortnum[i], PU_CACHE);
-    }
-
-  // unload tall percent
-  Z_ChangeTag(tallpercent, PU_CACHE);
-
-  // unload arms background
-  Z_ChangeTag(armsbg, PU_CACHE);
-
-  // unload gray #'s
-  for (i=0;i<6;i++)
-    Z_ChangeTag(arms[i][0], PU_CACHE);
-
-  // unload the key cards
-  for (i=0;i<NUMCARDS+3;i++)  //jff 2/23/98 unload double key patches too
-    Z_ChangeTag(keys[i], PU_CACHE);
-
-  Z_ChangeTag(sbar, PU_CACHE);
-
-  // killough 3/7/98: free each face background color
-  for (i=0;i<MAXPLAYERS;i++)
-    Z_ChangeTag(faceback[i], PU_CACHE);
-
-  for (i=0;i<ST_NUMFACES;i++)
-    if (faces[i])
-      Z_ChangeTag(faces[i], PU_CACHE);
-
-  // Note: nobody ain't seen no unloading of stminus yet. Dude.
-}
-
-void ST_unloadData(void)
-{
-  ST_unloadGraphics();
-}
-
 void ST_initData(void)
 {
   int i;
@@ -1143,13 +1095,7 @@ void ST_initData(void)
   st_firsttime = true;
   plyr = &players[displayplayer];            // killough 3/7/98
 
-  st_clock = 0;
-  st_chatstate = StartChatState;
-  st_gamestate = FirstPersonState;
-
   st_statusbaron = true;
-  st_oldchat = st_chat = false;
-  st_cursoron = false;
 
   st_faceindex = 0;
   st_palette = -1;
@@ -1188,7 +1134,7 @@ void ST_createWidgets(void)
                     ST_HEALTHX - distributed_delta,
                     ST_HEALTHY,
                     tallnum,
-                    &plyr->health,
+                    &st_health,
                     &st_statusbaron,
                     tallpercent);
 
@@ -1224,7 +1170,7 @@ void ST_createWidgets(void)
                     ST_ARMORX + distributed_delta,
                     ST_ARMORY,
                     tallnum,
-                    &plyr->armorpoints,
+                    &st_armor,
                     &st_statusbaron, tallpercent);
 
   // keyboxes 0-2
@@ -1320,7 +1266,7 @@ static void ST_MoveHud (void)
 {
     static int odelta = 0;
 
-    if (st_crispyhud && hud_distributed)
+    if (st_crispyhud && hud_active == 2)
         distributed_delta = WIDESCREENDELTA;
     else
         distributed_delta = 0;
@@ -1353,7 +1299,6 @@ void ST_Stop(void)
 
 void ST_Init(void)
 {
-  veryfirsttime = 0;
   ST_loadData();
   // killough 11/98: allocate enough for hires
   screens[4] = Z_Malloc(MAX_SCREENWIDTH*ST_HEIGHT*4, PU_STATIC, 0);
