@@ -1,7 +1,3 @@
-// Emacs style mode select   -*- C++ -*-
-//-----------------------------------------------------------------------------
-//
-// $Id: w_wad.c,v 1.20 1998/05/06 11:32:00 jim Exp $
 //
 //  Copyright (C) 1999 by
 //  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
@@ -16,11 +12,6 @@
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //  GNU General Public License for more details.
 //
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
-//  02111-1307, USA.
-//
 // DESCRIPTION:
 //      Handles WAD file header, directory, lump I/O.
 //
@@ -104,6 +95,7 @@ static void W_AddFile(const char *name) // killough 1/31/98: static, const
   int         startlump;
   filelump_t  *fileinfo, *fileinfo2free=NULL; //killough
   filelump_t  singleinfo;
+  boolean     is_single = false;
   char        *filename = strcpy(malloc(strlen(name)+5), name);
 
   NormalizeSlashes(AddDefaultExtension(filename, ".wad"));  // killough 11/98
@@ -135,6 +127,7 @@ static void W_AddFile(const char *name) // killough 1/31/98: static, const
       singleinfo.size = LONG(W_FileLength(handle));
       ExtractFileBase(filename, singleinfo.name);
       numlumps++;
+      is_single = true;
     }
   else
     {
@@ -173,9 +166,9 @@ static void W_AddFile(const char *name) // killough 1/31/98: static, const
         lump_p->size = LONG(fileinfo->size);
         lump_p->data = NULL;                        // killough 1/31/98
         lump_p->namespace = ns_global;              // killough 4/17/98
-        strncpy (lump_p->name, fileinfo->name, 8);
+        M_CopyLumpName(lump_p->name, fileinfo->name);
         // [FG] WAD file that contains the lump
-        lump_p->wad_file = name;
+        lump_p->wad_file = (is_single ? NULL : name);
       }
 
     free(fileinfo2free);      // killough
@@ -209,7 +202,7 @@ static void W_CoalesceMarkedResource(const char *start_marker,
       { // If this is the first start marker, add start marker to marked lumps
         if (!num_marked)
           {
-            memcpy(marked->name, start_marker, 8);
+            M_CopyLumpName(marked->name, start_marker);
             marked->size = 0;  // killough 3/20/98: force size to be 0
             marked->namespace = ns_global;        // killough 4/17/98
             num_marked = 1;
@@ -249,7 +242,7 @@ static void W_CoalesceMarkedResource(const char *start_marker,
     {
       lumpinfo[numlumps].size = 0;  // killough 3/20/98: force size to be 0
       lumpinfo[numlumps].namespace = ns_global;   // killough 4/17/98
-      memcpy(lumpinfo[numlumps++].name, end_marker, 8);
+      M_CopyLumpName(lumpinfo[numlumps++].name, end_marker);
     }
 }
 
@@ -457,7 +450,7 @@ void W_ReadLump(int lump, void *dest)
 //
 // killough 4/25/98: simplified
 
-void *W_CacheLumpNum(int lump, int tag)
+void *W_CacheLumpNum(int lump, pu_tag tag)
 {
 #ifdef RANGECHECK
   if ((unsigned)lump >= numlumps)
@@ -485,7 +478,7 @@ void *W_CacheLumpNum(int lump, int tag)
 // killough 4/22/98: make endian-independent, remove tab chars
 // haleyjd 01/21/05: rewritten to use stdio
 //
-void WritePredefinedLumpWad(const char *filename)
+void WriteLumpWad(const char *filename, const lumpinfo_t *lumps, const size_t num_lumps)
 {
    FILE *file;
    char *fn;
@@ -502,47 +495,59 @@ void WritePredefinedLumpWad(const char *filename)
    if((file = M_fopen(fn, "wb")))
    {
       wadinfo_t header = { "PWAD" };
-      size_t filepos = sizeof(wadinfo_t) + num_predefined_lumps * sizeof(filelump_t);
+      size_t filepos = sizeof(wadinfo_t) + num_lumps * sizeof(filelump_t);
       int i;
       
-      header.numlumps     = LONG(num_predefined_lumps);
+      header.numlumps     = LONG(num_lumps);
       header.infotableofs = LONG(sizeof(header));
       
       // write header
       fwrite(&header, 1, sizeof(header), file);
       
       // write directory
-      for(i = 0; i < num_predefined_lumps; i++)
+      for(i = 0; i < num_lumps; i++)
       {
          filelump_t fileinfo = { 0 };
          
          fileinfo.filepos = LONG(filepos);
-         fileinfo.size    = LONG(predefined_lumps[i].size);         
-         memcpy(fileinfo.name, predefined_lumps[i].name, 8);
+         fileinfo.size    = LONG(lumps[i].size);
+         M_CopyLumpName(fileinfo.name, lumps[i].name);
          
          fwrite(&fileinfo, 1, sizeof(fileinfo), file);
 
-         filepos += predefined_lumps[i].size;
+         filepos += lumps[i].size;
       }
       
       // write lumps
-      for(i = 0; i < num_predefined_lumps; i++)
-         fwrite(predefined_lumps[i].data, 1, predefined_lumps[i].size, file);
+      for(i = 0; i < num_lumps; i++)
+         fwrite(lumps[i].data, 1, lumps[i].size, file);
       
       fclose(file);
-      I_Error("Predefined lumps wad, %s written, exiting\n", filename);
+      I_Success("Internal lumps wad, %s written, exiting\n", filename);
    }
-   I_Error("Cannot open predefined lumps wad %s for output\n", filename);
+   I_Error("Cannot open internal lumps wad %s for output\n", filename);
    free(fn);
 }
 
+void WritePredefinedLumpWad(const char *filename)
+{
+    WriteLumpWad(filename, predefined_lumps, num_predefined_lumps);
+}
+
 // [FG] name of the WAD file that contains the lump
 const char *W_WadNameForLump (const int lump)
 {
-	return (lump >= 0 && lump < numlumps) ?
-	       (lumpinfo[lump].wad_file ?
-	       M_BaseName(lumpinfo[lump].wad_file) :
-	       "predefined") : "invalid";
+  if (lump < 0 || lump >= numlumps)
+    return "invalid";
+  else
+  {
+    const char *wad_file = lumpinfo[lump].wad_file;
+
+    if (wad_file)
+      return M_BaseName(wad_file);
+    else
+      return "lump";
+  }
 }
 
 boolean W_IsIWADLump (const int lump)
@@ -551,6 +556,12 @@ boolean W_IsIWADLump (const int lump)
 	       lumpinfo[lump].wad_file == wadfiles[0];
 }
 
+// check if lump is from WAD
+boolean W_IsWADLump (const int lump)
+{
+	return lump >= 0 && lump < numlumps && lumpinfo[lump].wad_file;
+}
+
 // [FG] avoid demo lump name collisions
 void W_DemoLumpNameCollision(char **name)
 {
